import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
// ignore: avoid_web_libraries_in_flutter
import 'dart:js' as js;

import 'package:uuid/uuid.dart';

import 'jsonrpc.dart';

class Common {
  static void setPageTitle(BuildContext context, String title) {
    SystemChrome.setApplicationSwitcherDescription(
        ApplicationSwitcherDescription(
      label: title,
      primaryColor:
          Theme.of(context).primaryColor.value, // This line is required
    ));
  }

  static MaterialColor createMaterialColor(Color color) {
    List strengths = <double>[.05];
    Map<int, Color> swatch = <int, Color>{};
    final int r = color.red, g = color.green, b = color.blue;

    for (int i = 1; i < 10; i++) {
      strengths.add(0.1 * i);
    }
    for (var strength in strengths) {
      final double ds = 0.5 - strength;
      swatch[(strength * 1000).round()] = Color.fromRGBO(
        r + ((ds < 0 ? r : (255 - r)) * ds).round(),
        g + ((ds < 0 ? g : (255 - g)) * ds).round(),
        b + ((ds < 0 ? b : (255 - b)) * ds).round(),
        1,
      );
    }
    return MaterialColor(color.value, swatch);
  }

  static String host = 'http://sapi.vinno.com/';

  static void init() {
    host = js.context.callMethod("getHost").toString();
  }

  static void jumpToUrl(String url) {
    js.context.callMethod('jumpToUrl', [url]);
  }

  static Future<String> createProgress() async {
    var progressId = const Uuid().v1();
    await JsonRpc().call(Common.host, 'ProgressService', 'AddOrUpdateProgress',
        [progressId, ""]);
    return progressId;
  }

  static Future<int> getProgress(String progressId) async {
    var result = await JsonRpc()
        .call(Common.host, 'ProgressService', 'GetProgress', [progressId]);
    var progressStr = result as String;
    if (progressStr.startsWith("error_")) {
      var errorStr = progressStr.substring(6);
      throw Exception(errorStr);
    } else {
      if (progressStr == "") {
        return 0;
      } else {
        return int.parse(progressStr);
      }
    }
  }

  static Future<String> deleteProgress(String progressId) async {
    await JsonRpc()
        .call(Common.host, 'ProgressService', 'DeleteProgress', [progressId]);
    return progressId;
  }
}
