// ignore_for_file: unnecessary_brace_in_string_interps

import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:intl/intl.dart';
import 'package:lpinyin/lpinyin.dart';

import 'common.dart';
import 'constants.dart';
import 'entities.dart';
import 'jsonrpc.dart';
import 'session.dart';
import 'viewmodels.dart';

class ControllerManager {
  static void init() {
    Get.put(LoginController());
    Get.put(MainController());
    Get.put(EmployeeController());
    Get.put(HolidayController());
    Get.put(StatisticsController());
    Get.put(DomainController());
    Get.put(EmailController());
    Get.put(AdminController());
    Get.put(MessageTemplateController());
    Get.put(LeaveQuotaController());
  }

  static T find<T>() {
    return Get.find<T>();
  }
}

class LoginController extends GetxController {
  var accountName = ''.obs;
  var password = ''.obs;

  Future login() async {
    var result = await JsonRpc().call(Common.host, 'LoginService', 'Login',
        [accountName.value, password.value]);
    if (result != null) {
      Session.currentSessionId = result.toString();
      Session.currentAccountName = accountName.value;
      Session.isAdmin = true;
    } else {
      throw Exception('登录失败：未知错误。');
    }
  }
}

class MainController extends GetxController {
  Rx<String> currentModule = Rx<String>(ModuleNames.employee);

  Future logOff() async {
    var result = await JsonRpc().call(
        Common.host, 'LoginService', 'Logoff', [Session.currentSessionId]);
    if (result != null) {
      Session.currentSessionId = '';
    }
  }

  void showModule(String moduleName) {
    currentModule.value = moduleName;
  }
}

class EmployeeController extends GetxController {
  DepartmentViewModel? selectedDepartment;

  Rx<DepartmentViewModel> currentParentDepartment =
      Rx(DepartmentViewModel(Department(1, 'VINNO', 0), null));

  Rx<List<DepartmentViewModel>> departments = Rx([]);

  String filterKey = '';

  RxBool filterMode = RxBool(false);

  Rx<List<EmployeeViewModel>> employees = Rx([]);

  Rx<List<EmployeeViewModel>> filteredEmployees = Rx([]);

  Future<int> getEmployeeCount() async {
    var result = await JsonRpc().call(Common.host, 'EmployeeService',
        'GetEmployeeCount', [Session.currentSessionId]);
    return result as int;
  }

  Future<int> getStruggleEmployeeCount() async {
    var result = await JsonRpc().call(Common.host, 'EmployeeService',
        'GetStruggleEmployeeCount', [Session.currentSessionId]);
    return result as int;
  }

  Future<Employee> getEmployeeByName(String employeeName) async {
    var result = await JsonRpc().call(Common.host, 'EmployeeService',
        'GetByName', [Session.currentSessionId, employeeName]);
    var map = result as Map<String, dynamic>;
    var employee = Employee.fromJosn(map);
    return employee;
  }

  Future updateEmployeeDepartment(EmployeeViewModel employee) async {
    employee.data.isReviewer = employee.isReviewer.value;
    await JsonRpc().call(Common.host, 'EmployeeService',
        'UpdateEmployeeDepartment', [Session.currentSessionId, employee.data]);
    var tempList = employees.value.toList();
    tempList.remove(employee);
    employees.value = tempList;
  }

  Future updateEmployeeIsReviewer(EmployeeViewModel employee) async {
    employee.data.isReviewer = employee.isReviewer.value;
    await JsonRpc().call(Common.host, 'EmployeeService',
        'UpdateEmployeeIsReviewer', [Session.currentSessionId, employee.data]);
  }

  Future updateEmployeeIsStruggle(EmployeeViewModel employee) async {
    employee.data.isStruggle = employee.isStruggle.value;
    await JsonRpc().call(Common.host, 'EmployeeService',
        'UpdateEmployeeIsStruggle', [Session.currentSessionId, employee.data]);
  }

  Future loadDepartments(DepartmentViewModel parent) async {
    List<DepartmentViewModel> loadedDepartments = [];
    var result = await JsonRpc().call(Common.host, 'DepartmentService',
        'GetChildDepartments', [Session.currentSessionId, parent.data.id]);
    if (result != null) {
      List list = result as List;
      for (var item in list) {
        var map = item as Map<String, dynamic>;
        var department = Department.fromJosn(map);
        loadedDepartments.add(DepartmentViewModel(department, parent));
        await Future.delayed(const Duration(milliseconds: 1));
      }
    }
    departments.value = loadedDepartments;
    currentParentDepartment.value = parent;
  }

  Future<List<EmployeeViewModel>> loadEmployees2(int departmentId) async {
    filterMode.value = false;
    List<EmployeeViewModel> loadedEmployees = [];
    var result = await JsonRpc().call(Common.host, 'EmployeeService',
        'GetManyByDepartmentId', [Session.currentSessionId, departmentId]);
    if (result != null) {
      List list = result as List;
      for (var item in list) {
        var map = item as Map<String, dynamic>;
        var employee = Employee.fromJosn(map);
        loadedEmployees.add(EmployeeViewModel(employee));
        await Future.delayed(const Duration(milliseconds: 1));
      }
    }
    return loadedEmployees;
  }

  Future loadEmployees(DepartmentViewModel department) async {
    filterMode.value = false;
    List<EmployeeViewModel> loadedEmployees = [];
    var result = await JsonRpc().call(
        Common.host,
        'EmployeeService',
        'GetManyByDepartmentId',
        [Session.currentSessionId, department.data.id]);
    if (result != null) {
      List list = result as List;
      for (var item in list) {
        var map = item as Map<String, dynamic>;
        var employee = Employee.fromJosn(map);
        loadedEmployees.add(EmployeeViewModel(employee));
        await Future.delayed(const Duration(milliseconds: 1));
      }
    }
    employees.value = loadedEmployees;
    selectedDepartment = department;
  }

  Future deleteEmployee(EmployeeViewModel employee) async {
    await JsonRpc().call(Common.host, 'EmployeeService', 'Delete',
        [Session.currentSessionId, employee.data.id]);
    var temp1 = employees.value.toList();
    var tmep2 = filteredEmployees.value.toList();

    temp1.remove(employee);
    tmep2.remove(employee);

    employees.value = temp1;
    filteredEmployees.value = tmep2;
  }

  Future<List<Employee>> loadReviewers() async {
    List<Employee> reviewers = [];
    var result = await JsonRpc().call(Common.host, 'EmployeeService',
        'GetReviewers', [Session.currentSessionId]);
    if (result != null) {
      List list = result as List;
      for (var item in list) {
        var map = item as Map<String, dynamic>;
        var employee = Employee.fromJosn(map);
        reviewers.add(employee);
      }
    }
    return reviewers;
  }

  Future sendTextMessage(String userId, String message) async {
    await JsonRpc().call(
        Common.host, 'DingTalkService', 'SendTextMessage', [userId, message]);
  }

  void filterEmployees() {
    if (filterKey != '') {
      filterMode.value = true;
      filteredEmployees.value = employees.value
          .where((employee) => employee.data.name.contains(filterKey))
          .toList();
    } else {
      filterMode.value = false;
    }
  }
}

class StatisticsController extends GetxController {
  RxBool filterMode = RxBool(false);

  String employeeFilter = '';
  String departmentFilter = '';

  List<String> currentDepartmentNameList = [];

  StatisticsTable? selectedTable;

  Rx<List<WorkingDayInfoViewModel>> workingDayInfos = Rx([]);

  WorkingDayStatisticsViewModel? selectedWorkingDayStatistics;

  Rx<List<WorkingDayStatisticsViewModel>> filteredWorkingDayStatisticsList =
      Rx([]);

  Rx<List<WorkingDayStatisticsViewModel>> workingDayStatisticsList = Rx([]);

  Rx<List<StatisticsTableViewModel>> statisticsTables = Rx([]);

  StatisticsController();

  Future _loadWorkingDayStatisticsListList(List<String> ids) async {
    List<WorkingDayStatisticsViewModel> loadedEmployeeStatisticsList = [];
    currentDepartmentNameList.clear();
    currentDepartmentNameList.add('');
    employeeFilter = '';
    departmentFilter = '';

    var result = await JsonRpc().call(
        Common.host,
        'WorkingDayStatisticsService',
        'GetMany',
        [Session.currentSessionId, ids]);

    if (result != null) {
      List list = result as List;
      for (var item in list) {
        var map = item as Map<String, dynamic>;
        var workingDayStatistics = WorkingDayStatistics.fromJosn(map);
        loadedEmployeeStatisticsList
            .add(WorkingDayStatisticsViewModel(workingDayStatistics));
        if (!currentDepartmentNameList
            .contains(workingDayStatistics.departmentName)) {
          currentDepartmentNameList.add(workingDayStatistics.departmentName);
        }
      }
    }
    workingDayStatisticsList.value = loadedEmployeeStatisticsList;
  }

  Future<List<WorkingDayStatistics>> getWorkingDayStatisticsList(
      List<String> ids) async {
    List<WorkingDayStatistics> rsuultList = [];
    var result = await JsonRpc().call(
        Common.host,
        'WorkingDayStatisticsService',
        'GetMany',
        [Session.currentSessionId, ids]);

    if (result != null) {
      List list = result as List;
      for (var item in list) {
        var map = item as Map<String, dynamic>;
        var workingDayStatistics = WorkingDayStatistics.fromJosn(map);
        rsuultList.add(workingDayStatistics);
      }
    }
    return rsuultList;
  }

  Future<ViewCode> _loadViewCode(String code) async {
    var result =
        await JsonRpc().call(Common.host, 'ViewCodeService', 'Get', [code]);
    return ViewCode.fromJosn(result);
  }

  Future<int> check(String email) async {
    var now = DateTime.now();
    var today = DateTime(now.year, now.month, now.day);
    var start = today;
    if (start.day < 26) {
      if (start.month == 1) {
        start = DateTime(start.year - 1, 12, 26);
      } else {
        start = DateTime(start.year, start.month - 1, 26);
      }
    } else {
      start = DateTime(start.year, start.month, 26);
    }
    var startStr = DateFormat('yyyy-MM-dd HH:mm:ss').format(start);
    var endStr = DateFormat('yyyy-MM-dd HH:mm:ss').format(today);
    var result = await JsonRpc().call(
        Common.host,
        'WorkingDayStatisticsService',
        'CheckEmployeeStatistics',
        [email, startStr, endStr]);
    return result as int;
  }

  Future<int> checkPerformance(
      String email, DateTime start, DateTime end) async {
    var startStr = DateFormat('yyyy-MM-dd HH:mm:ss').format(start);
    var endStr = DateFormat('yyyy-MM-dd HH:mm:ss').format(end);
    var result = await JsonRpc().call(
        Common.host,
        'WorkingDayStatisticsService',
        'CheckEmployeePerformanceHours',
        [email, startStr, endStr]);
    return result as int;
  }

  Future selectViewCode(String code) async {
    var viewCode = await _loadViewCode(code);
    if (viewCode.expiredTime.isBefore(DateTime.now())) {
      throw Exception('链接已过期');
    }
    await _loadWorkingDayStatisticsListList(viewCode.workingDayStatisticsIds);
  }

  Future createViewCode(ViewCode viewCode) async {
    await JsonRpc().call(Common.host, 'ViewCodeService', 'Add',
        [Session.currentSessionId, viewCode]);
  }

  Future createAndSendViewCode(String receiver, String cc,
      String contentTemplate, ViewCode viewCode) async {
    await createViewCode(viewCode);
    await JsonRpc().call(Common.host, 'ViewCodeService', 'SendViewCode', [
      Session.currentSessionId,
      selectedTable!.title,
      receiver,
      cc,
      contentTemplate,
      viewCode
    ]);
  }

  Future<String> createAndDownloadReport() async {
    var selectedEmployeeStatisticsList = workingDayStatisticsList.value
        .where((element) => element.isSelected.value)
        .map((e) => e.data.id)
        .toList();
    var reportTitle = 'Report';
    if (selectedTable != null) {
      reportTitle = selectedTable!.title;
    }
    var result = await JsonRpc().call(
        Common.host, 'StatisticsTableService', 'CreateAndDownloadReport', [
      Session.currentSessionId,
      reportTitle,
      selectedEmployeeStatisticsList
    ]);
    return result.toString();
  }

  Future updateWorkingDayInfo(WorkingDayInfo workingDayInfo) async {
    await JsonRpc().call(Common.host, 'WorkingDayService', 'Update',
        [Session.currentSessionId, workingDayInfo]);
  }

  Future updateWorkingDayStatistics(WorkingDayStatistics wds) async {
    await JsonRpc().call(Common.host, 'WorkingDayStatisticsService', 'Update',
        [Session.currentSessionId, wds]);
  }

  Future updateWorkingDayStatisticsComment(
      String workingDayStatisticsId, String comment) async {
    await JsonRpc().call(
        Common.host,
        'WorkingDayStatisticsService',
        'UpdateComment',
        [Session.currentSessionId, workingDayStatisticsId, comment]);
  }

  Future selectTable(StatisticsTable? table) async {
    if (table == null) {
      workingDayStatisticsList.value = [];
      selectedTable = null;
    } else {
      await _loadWorkingDayStatisticsListList(table.workingDayStatisticsIds);
      selectedTable = table;
    }
  }

  Future _loadWorkDayInfos(WorkingDayStatistics workingDayStatistics) async {
    List<WorkingDayInfoViewModel> loadedWorkingDayInfos = [];
    var result =
        await JsonRpc().call(Common.host, 'WorkingDayService', 'GetMany', [
      Session.currentSessionId,
      workingDayStatistics.employee.id,
      workingDayStatistics.id
    ]);
    if (result != null) {
      List list = result as List;
      for (var item in list) {
        var map = item as Map<String, dynamic>;
        var workingDayInfo = WorkingDayInfo.fromJosn(map);
        loadedWorkingDayInfos.add(WorkingDayInfoViewModel(workingDayInfo));
      }
    }
    workingDayInfos.value = loadedWorkingDayInfos;
  }

  Future selectWorkingDayStatistics(
      WorkingDayStatisticsViewModel? workingDayStatistics) async {
    if (workingDayStatistics == null) {
      workingDayInfos.value = [];
      selectedWorkingDayStatistics = null;
    } else {
      await _loadWorkDayInfos(workingDayStatistics.data);
      selectedWorkingDayStatistics = workingDayStatistics;
    }
  }

  void filterStatisticsList() {
    filterMode.value = true;
    for (var workingDayStatistics in workingDayStatisticsList.value) {
      workingDayStatistics.isSelected.value = false;
    }

    if (departmentFilter != "") {
      filteredWorkingDayStatisticsList.value = workingDayStatisticsList.value
          .where((element) =>
              element.data.departmentName.contains(departmentFilter) &&
              (element.data.employee.name
                      .toLowerCase()
                      .contains(employeeFilter.toLowerCase()) ||
                  PinyinHelper.getShortPinyin(
                          element.data.employee.name.toLowerCase())
                      .contains(employeeFilter.toLowerCase())))
          .toList();
    } else {
      filteredWorkingDayStatisticsList.value = workingDayStatisticsList.value
          .where((element) =>
              element.data.employee.name
                  .toLowerCase()
                  .contains(employeeFilter.toLowerCase()) ||
              PinyinHelper.getShortPinyin(
                      element.data.employee.name.toLowerCase())
                  .contains(employeeFilter.toLowerCase()))
          .toList();
    }
  }

  void reset() {
    filterMode.value = false;
    employeeFilter = "";
    departmentFilter = "";
    for (var workingDayStatistics in workingDayStatisticsList.value) {
      workingDayStatistics.isSelected.value = false;
    }
  }

  Future<List<StatisticsTable>> getStatisticsTables() async {
    List<StatisticsTable> tables = [];
    var result = await JsonRpc().call(Common.host, 'StatisticsTableService',
        'GetMany', [Session.currentSessionId]);
    if (result != null) {
      List list = result as List;
      for (var item in list) {
        var map = item as Map<String, dynamic>;
        var statisticsTables = StatisticsTable.fromJosn(map);
        tables.add(statisticsTables);
      }
    }
    return tables;
  }

  Future loadStatisticsTables() async {
    List<StatisticsTableViewModel> loadedStatisticsTables = [];
    var result = await JsonRpc().call(Common.host, 'StatisticsTableService',
        'GetMany', [Session.currentSessionId]);
    if (result != null) {
      List list = result as List;
      for (var item in list) {
        var map = item as Map<String, dynamic>;
        var table = StatisticsTable.fromJosn(map);
        loadedStatisticsTables.add(StatisticsTableViewModel(table));
      }
    }
    statisticsTables.value = loadedStatisticsTables;
  }

  Future addStatisticsTable(
      String title, String start, String end, int requiredReachedCount) async {
    await JsonRpc().call(Common.host, 'StatisticsTableService', 'Add',
        [Session.currentSessionId, title, start, end, requiredReachedCount]);
    await loadStatisticsTables();
  }

  Future addStatisticsTable2(String title, String start, String end,
      int requiredReachedCount, String progressId) async {
    await JsonRpc().call(Common.host, 'StatisticsTableService', 'Add2', [
      Session.currentSessionId,
      title,
      start,
      end,
      requiredReachedCount,
      progressId
    ]);
    await loadStatisticsTables();
  }

  Future deleteStatisticsTable(String id) async {
    await JsonRpc().call(Common.host, 'StatisticsTableService', 'Delete',
        [Session.currentSessionId, id]);
    await loadStatisticsTables();
  }
}

class EmailController extends GetxController {
  EmailSetting? setting;

  EmailController();

  Future getEmailSetting() async {
    var result = await JsonRpc().call(Common.host, 'EmailService',
        'GetEmailSetting', [Session.currentSessionId]);
    setting = EmailSetting.fromJosn(result as Map<String, dynamic>);
  }

  Future addOrUpdateEmailSetting(EmailSetting setting) async {
    await JsonRpc().call(Common.host, 'EmailService', 'AddOrUpdateEmailSetting',
        [Session.currentSessionId, setting]);
  }
}

class HolidayController extends GetxController {
  Rx<DateTime> currentStart = Rx<DateTime>(DateTime.now());
  Rx<DateTime> currentEnd = Rx<DateTime>(DateTime.now());

  Rx<List<HolidayInfo>> holidays = Rx([]);

  HolidayController() {
    var now = DateTime.now();
    var daysInMonth = DateUtils.getDaysInMonth(now.year, now.month);
    currentStart.value = DateUtils.addDaysToDate(now, -now.day + 1);
    currentEnd.value = DateUtils.addDaysToDate(now, daysInMonth - now.day + 1);
  }

  void moveToPreviewMonth() {
    var start = DateUtils.addMonthsToMonthDate(currentStart.value, -1);
    var daysInMonth = DateUtils.getDaysInMonth(start.year, start.month);
    currentStart.value = DateUtils.addDaysToDate(start, -start.day + 1);
    currentEnd.value =
        DateUtils.addDaysToDate(start, daysInMonth - start.day + 1);
    currentStart.value = start;
  }

  void moveToNextMonth() {
    var start = DateUtils.addMonthsToMonthDate(currentStart.value, 1);
    var daysInMonth = DateUtils.getDaysInMonth(start.year, start.month);
    currentStart.value = DateUtils.addDaysToDate(start, -start.day + 1);
    currentEnd.value =
        DateUtils.addDaysToDate(start, daysInMonth - start.day + 1);
    currentStart.value = start;
  }

  Future load2YearsHolidays() async {
    List<HolidayInfo> loadedHolidays = [];
    var result = await JsonRpc().call(
        Common.host, 'HolidayService', 'GetMany', [Session.currentSessionId]);
    if (result != null) {
      List list = result as List;
      for (var item in list) {
        var map = item as Map<String, dynamic>;
        var holiday = HolidayInfo.fromJosn(map);
        loadedHolidays.add(holiday);
      }
    }
    holidays.value = loadedHolidays;
  }

  Future addHoliday(HolidayInfo holiday) async {
    var temp = holidays.value.toList();
    await JsonRpc().call(Common.host, 'HolidayService', 'Add',
        [Session.currentSessionId, holiday]);
    var first = temp.firstWhereOrNull(
        (element) => element.holiday.isAfter(holiday.holiday));
    if (first == null) {
      temp.insert(0, holiday);
    } else {
      temp.insert(temp.indexOf(first) + 1, holiday);
    }
    holidays.value = temp;
  }

  Future deleteHoliday(HolidayInfo holiday) async {
    var temp = holidays.value.toList();
    await JsonRpc().call(Common.host, 'HolidayService', 'Delete',
        [Session.currentSessionId, holiday]);
    temp.remove(holiday);
    holidays.value = temp;
  }
}

class AdminController extends GetxController {
  AdminController();

  Future updateAdminPasword(String password) async {
    await JsonRpc().call(Common.host, 'AdminService', 'Update',
        [Session.currentSessionId, Session.currentAccountName, password]);
  }

  Future<List<String>> getAdminIds() async {
    var result = await JsonRpc().call(Common.host, 'AdminService',
        'GetAdminIds', [Session.currentSessionId]) as List<dynamic>;
    return result.map((e) => e.toString()).toList();
  }
}

class DomainController extends GetxController {
  DomainSetting? setting;

  DomainController();

  Future getDomainSetting() async {
    var result = await JsonRpc().call(Common.host, 'DomainService',
        'GetDomainSetting', [Session.currentSessionId]);
    setting = DomainSetting.fromJosn(result as Map<String, dynamic>);
  }

  Future addOrUpdateDomainSetting(DomainSetting setting) async {
    await JsonRpc().call(Common.host, 'DomainService',
        'AddOrUpdateDomainSetting', [Session.currentSessionId, setting]);
  }
}

class MessageTemplateController extends GetxController {
  List<MessageTemplate> templates = [];

  MessageTemplateController();

  Future getMessageTemplate() async {
    var result = await JsonRpc().call(
        Common.host,
        'MessageTemplateService',
        'GetMessageTemplate',
        [Session.currentSessionId, 'ReviewMessageTemplate']);
    var template = MessageTemplate.fromJosn(result as Map<String, dynamic>);
    templates.add(template);
  }

  Future addOrUpdateMessageTemplate(MessageTemplate template) async {
    await JsonRpc().call(Common.host, 'MessageTemplateService',
        'AddOrUpdateGetMessageTemplate', [Session.currentSessionId, template]);
  }
}

class LeaveQuotaController extends GetxController {
  LeaveQuotaController();

  Future<List<LeaveQuota>> getLeaveQuotas(String email, int leaveType) async {
    List<LeaveQuota> leaveQuotas = [];
    var result = await JsonRpc().call(
        Common.host, 'EmployeeService', 'GetLeaveQuotas', [email, leaveType]);
    if (result != null) {
      List list = result as List;
      for (var item in list) {
        var map = item as Map<String, dynamic>;
        var leaveQuota = LeaveQuota.fromJosn(map);
        leaveQuotas.add(leaveQuota);
      }
    }
    return leaveQuotas;
  }
}
