import 'package:fl_chart/fl_chart.dart';
import 'package:flutter/material.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';
import 'package:get/get.dart';
import 'package:intl/intl.dart';
import 'package:sss/controllers.dart';
import 'package:sss/entities.dart';
import 'package:sss/layout.dart';

class DepartmentStruggleInfo {
  String name;
  int employeeCount;
  int struggleEmployeeCount;
  int avgReachedCount;
  DepartmentStruggleInfo(this.name, this.employeeCount,
      this.struggleEmployeeCount, this.avgReachedCount);
}

class DashboardModule extends StatelessWidget {
  int _struggleEmployeeCount = 0;
  int _unStruggleEmployeeCount = 0;
  List<StatisticsTable> _statisticsTables = [];
  List<WorkingDayStatistics> _orderedLastWorkingDayStatisticsList = [];

  final Map<int, String> _includedDepartments = {
    51669094: 'SW',
    679282135: 'RD3',
    102033510: 'RD2',
    102097343: 'RD1',
    102219320: 'Common_Tech',
    51669098: 'TRANSDUCER',
    120752881: 'Research',
    51669102: 'IDS',
    678380206: 'Surgical Visualization',
    843456383: 'LSI R&D'
  };

  String _statTitle = '部门加班数据统计';

  final List<DepartmentStruggleInfo> _departmentStruggleInfos = [];

  DashboardModule({super.key});

  Future<bool> _loadHomeData(BuildContext context) async {
    var ec = Get.find<EmployeeController>();
    var sc = Get.find<StatisticsController>();
    EasyLoading.show(status: '载入中...');
    try {
      var employeeCount = await ec.getEmployeeCount();
      _struggleEmployeeCount = await ec.getStruggleEmployeeCount();
      _unStruggleEmployeeCount = employeeCount - _struggleEmployeeCount;

      _statisticsTables = (await sc.getStatisticsTables()).reversed.toList();
      if (_statisticsTables.isNotEmpty) {
        var ids = _statisticsTables.last.workingDayStatisticsIds;
        var list = await sc.getWorkingDayStatisticsList(ids);
        if (list.isNotEmpty) {
          list.sort((a, b) =>
              -a.adjustedReachedCount.compareTo(b.adjustedReachedCount));
          _orderedLastWorkingDayStatisticsList = list.take(10).toList();
        }
      }
      List<WorkingDayStatistics> workingDayStatisticsList = [];
      var statisticsTables = (await sc.getStatisticsTables()).reversed.toList();
      if (statisticsTables.isNotEmpty) {
        var ids = statisticsTables.last.workingDayStatisticsIds;
        _statTitle =
            '${statisticsTables.last.createTime.year}年${statisticsTables.last.createTime.month}月部门加班数据统计';
        workingDayStatisticsList = await sc.getWorkingDayStatisticsList(ids);
      }

      for (var departmentId in _includedDepartments.keys) {
        var employees = await ec.loadEmployees2(departmentId);
        var employeeCount = employees.length;
        var struggleEmployees =
            employees.where((x) => x.data.isStruggle).toList();
        var struggleEmployeeCount = struggleEmployees.length;
        var reachedCount = 0;
        if (workingDayStatisticsList.isNotEmpty) {
          for (var employee in struggleEmployees) {
            var workingDayStatistics = workingDayStatisticsList
                .firstWhereOrNull((e) => e.employee.id == employee.data.id);
            if (workingDayStatistics != null) {
              reachedCount += workingDayStatistics.adjustedReachedCount;
            }
          }
        }
        if (employeeCount > 0) {
          var avgReachedCount = 0;
          if (struggleEmployeeCount > 0) {
            avgReachedCount = (reachedCount / struggleEmployeeCount).round();
          }
          _departmentStruggleInfos.add(DepartmentStruggleInfo(
              _includedDepartments[departmentId]!,
              employeeCount,
              struggleEmployeeCount,
              avgReachedCount));
        }
      }
      _departmentStruggleInfos
          .sort((a, b) => a.avgReachedCount.compareTo(b.avgReachedCount));
      EasyLoading.dismiss();
      return true;
    } catch (e) {
      EasyLoading.dismiss();
      EasyLoading.showError('载入数据失败', duration: const Duration(seconds: 1));
      return false;
    }
  }

  @override
  Widget build(context) {
    return FutureBuilder(
        future: _loadHomeData(context),
        builder: ((context, snapshot) {
          if (snapshot.hasData && snapshot.data as bool == true) {
            return SingleChildScrollView(
                controller: ScrollController(),
                child: Column(
                  children: [
                    Container(
                        height: 480,
                        padding: const EdgeInsets.all(10),
                        alignment: Alignment.topLeft,
                        child: Column(children: [
                          Text(_statTitle,
                              style: const TextStyle(
                                  fontSize: 24, fontWeight: FontWeight.bold)),
                          const SizedBox(height: 10),
                          Expanded(child: _createDepStruggleRanking())
                        ])),
                    Container(
                        padding: const EdgeInsets.all(30),
                        alignment: Alignment.topLeft,
                        child: Wrap(spacing: 48, runSpacing: 48, children: [
                          Obx(() => SizedBox(
                              width: LayoutUtils.isDesktopMode.value
                                  ? ((LayoutUtils.width.value - 384) / 3)
                                  : LayoutUtils.width.value,
                              height: LayoutUtils.isDesktopMode.value
                                  ? ((LayoutUtils.width.value - 384) / 3)
                                  : LayoutUtils.width.value,
                              child: Column(
                                children: [
                                  const Text('加班人员占比',
                                      style: TextStyle(
                                          fontSize: 24,
                                          fontWeight: FontWeight.bold)),
                                  const SizedBox(height: 10),
                                  Expanded(child: _createStruggleEmployeePec())
                                ],
                              ))),
                          Obx(() => SizedBox(
                                width: LayoutUtils.isDesktopMode.value
                                    ? ((LayoutUtils.width.value - 384) / 3)
                                    : LayoutUtils.width.value,
                                height: LayoutUtils.isDesktopMode.value
                                    ? ((LayoutUtils.width.value - 384) / 3)
                                    : LayoutUtils.width.value,
                                child: Column(
                                  children: [
                                    const Text('加班参与人数趋势',
                                        style: TextStyle(
                                            fontSize: 24,
                                            fontWeight: FontWeight.bold)),
                                    const SizedBox(height: 10),
                                    Expanded(
                                        child: _createStruggleEmployeeTrend())
                                  ],
                                ),
                              )),
                          Obx(() => SizedBox(
                                width: LayoutUtils.isDesktopMode.value
                                    ? ((LayoutUtils.width.value - 384) / 3)
                                    : LayoutUtils.width.value,
                                height: LayoutUtils.isDesktopMode.value
                                    ? ((LayoutUtils.width.value - 384) / 3)
                                    : LayoutUtils.width.value,
                                child: Column(
                                  children: [
                                    const Text('加班达标排行',
                                        style: TextStyle(
                                            fontSize: 24,
                                            fontWeight: FontWeight.bold)),
                                    const SizedBox(height: 10),
                                    Expanded(
                                        child: _createStruggleEmployeeRanking())
                                  ],
                                ),
                              )),
                          Obx(() => LayoutUtils.isDesktopMode.value
                              ? const SizedBox()
                              : const SizedBox(height: 64))
                        ]))
                  ],
                ));
          } else {
            return const Center();
          }
        }));
  }

  Widget getDepStruggleRankingTitles(double value, TitleMeta meta) {
    var style = LayoutUtils.isDesktopMode.value
        ? const TextStyle(
            color: Colors.black,
            fontSize: 12,
          )
        : const TextStyle(
            color: Colors.black,
            fontSize: 8,
          );
    var info = _departmentStruggleInfos[value.toInt()];
    var pec = (info.struggleEmployeeCount / info.employeeCount * 100).round();
    var matrix = Matrix4.rotationZ(-20);
    matrix.add(Matrix4.translationValues(10, 90, 0));
    return Obx(() => Container(
        child: Column(
          children: [
            Text(info.name, style: style),
            Text(
                LayoutUtils.isDesktopMode.value
                    ? '加班率: $pec%'
                    : '加班率: $pec%(${info.struggleEmployeeCount}/${info.employeeCount})',
                style: style),
            LayoutUtils.isDesktopMode.value
                ? Text('${info.struggleEmployeeCount}/${info.employeeCount}',
                    style: style)
                : const SizedBox(width: 0, height: 0),
          ],
        ),
        transform: LayoutUtils.isDesktopMode.value ? null : matrix));
  }

  List<BarChartGroupData> _createDepStruggleRankingTitlesBarGroups() {
    List<BarChartGroupData> barChartGroupDataList = [];
    var index = 0;
    for (var item in _departmentStruggleInfos) {
      barChartGroupDataList.add(BarChartGroupData(
        x: index,
        barRods: [
          BarChartRodData(
            toY: item.avgReachedCount.toDouble(),
          )
        ],
        showingTooltipIndicators: [0],
      ));
      index++;
    }
    return barChartGroupDataList;
  }

  Widget _createDepStruggleRanking() {
    if (_departmentStruggleInfos.isNotEmpty) {
      return BarChart(
        BarChartData(
          //barTouchData: barTouchData,
          titlesData: FlTitlesData(
            show: true,
            bottomTitles: AxisTitles(
              sideTitles: SideTitles(
                showTitles: true,
                reservedSize: 48,
                getTitlesWidget: getDepStruggleRankingTitles,
              ),
            ),
            leftTitles: AxisTitles(
              sideTitles: SideTitles(showTitles: false),
            ),
            topTitles: AxisTitles(
              sideTitles: SideTitles(showTitles: false),
            ),
            rightTitles: AxisTitles(
              sideTitles: SideTitles(showTitles: false),
            ),
          ),
          //borderData: borderData,
          barGroups: _createDepStruggleRankingTitlesBarGroups(),
          gridData: FlGridData(show: false),
          alignment: BarChartAlignment.spaceAround,
          maxY: 32,
        ),
        swapAnimationDuration: const Duration(milliseconds: 150), // Optional
        swapAnimationCurve: Curves.linear, // Optional
      );
    }
    return const Center();
  }

  Widget getTitles(double value, TitleMeta meta) {
    var style = LayoutUtils.isDesktopMode.value
        ? const TextStyle(
            color: Colors.black,
            fontSize: 12,
          )
        : const TextStyle(
            color: Colors.black,
            fontSize: 10,
          );
    String text =
        _orderedLastWorkingDayStatisticsList[value.toInt()].employee.name;
    var matrix = Matrix4.rotationZ(-20);
    matrix.add(Matrix4.translationValues(10, 90, 0));
    return Container(
        child: Center(child: Text(text, style: style)), transform: matrix);
  }

  List<BarChartGroupData> _createBarGroups() {
    List<BarChartGroupData> barChartGroupDataList = [];
    var index = 0;
    for (var item in _orderedLastWorkingDayStatisticsList) {
      barChartGroupDataList.add(BarChartGroupData(
        x: index,
        barRods: [
          BarChartRodData(
            toY: item.adjustedReachedCount.toDouble(),
          )
        ],
        showingTooltipIndicators: [0],
      ));
      index++;
    }
    return barChartGroupDataList;
  }

  Widget _createStruggleEmployeeRanking() {
    if (_orderedLastWorkingDayStatisticsList.isNotEmpty) {
      return BarChart(
        BarChartData(
          //barTouchData: barTouchData,
          titlesData: FlTitlesData(
            show: true,
            bottomTitles: AxisTitles(
              sideTitles: SideTitles(
                showTitles: true,
                reservedSize: 16,
                getTitlesWidget: getTitles,
              ),
            ),
            leftTitles: AxisTitles(
              sideTitles: SideTitles(showTitles: false),
            ),
            topTitles: AxisTitles(
              sideTitles: SideTitles(showTitles: false),
            ),
            rightTitles: AxisTitles(
              sideTitles: SideTitles(showTitles: false),
            ),
          ),
          //borderData: borderData,
          barGroups: _createBarGroups(),
          gridData: FlGridData(show: false),
          alignment: BarChartAlignment.spaceAround,
          maxY: 44,
        ),
        swapAnimationDuration: const Duration(milliseconds: 150), // Optional
        swapAnimationCurve: Curves.linear, // Optional
      );
    }
    return const Center();
  }

  Widget _createStruggleEmployeePec() {
    var struggleEmployeePec = (_struggleEmployeeCount /
        (_struggleEmployeeCount + _unStruggleEmployeeCount));
    var unStruggleEmployeePec = (_unStruggleEmployeeCount /
        (_struggleEmployeeCount + _unStruggleEmployeeCount));
    return PieChart(
      PieChartData(
          // read about it in the PieChartData section
          sections: [
            PieChartSectionData(
                value: struggleEmployeePec,
                title: '加班人员占比 ${(struggleEmployeePec * 100).round()}%',
                radius: 150,
                color: Colors.green,
                titleStyle: const TextStyle(color: Colors.white, fontSize: 12)),
            PieChartSectionData(
                value: unStruggleEmployeePec,
                title: '不加班人员占比 ${(unStruggleEmployeePec * 100).round()}%',
                radius: 150,
                color: Colors.red,
                titleStyle: const TextStyle(color: Colors.white, fontSize: 12)),
          ],
          pieTouchData: PieTouchData(enabled: true)),
      swapAnimationDuration: const Duration(milliseconds: 150), // Optional
      swapAnimationCurve: Curves.linear, // Optional
    );
  }

  Widget createBottomTitleWidgets(double value, TitleMeta meta) {
    var style = LayoutUtils.isDesktopMode.value
        ? const TextStyle(
            color: Colors.black,
            fontSize: 12,
          )
        : const TextStyle(
            color: Colors.black,
            fontSize: 10,
          );
    if (value.toInt() < _statisticsTables.length) {
      var matrix = Matrix4.rotationZ(-20);
      matrix.add(Matrix4.translationValues(10, 80, 0));
      return Container(
          child: Text(
              DateFormat('yy/MM/dd')
                  .format(_statisticsTables[value.toInt()].createTime),
              style: style),
          transform: matrix);
    }
    return const Padding(child: Text(''), padding: EdgeInsets.only(top: 10.0));
  }

  Widget createLeftTitleWidgets(double value, TitleMeta meta) {
    var style = LayoutUtils.isDesktopMode.value
        ? const TextStyle(
            color: Colors.black,
            fontSize: 12,
          )
        : const TextStyle(
            color: Colors.black,
            fontSize: 10,
          );
    return Text(value.toInt().toString(),
        style: style, textAlign: TextAlign.center);
  }

  Widget _createStruggleEmployeeTrend() {
    var maxValue = _struggleEmployeeCount + _unStruggleEmployeeCount;
    List<FlSpot> spots = [];
    var index = 0;
    for (var collection in _statisticsTables) {
      spots.add(FlSpot(index.toDouble(),
          collection.workingDayStatisticsIds.length.toDouble()));
      index++;
    }
    return LineChart(
      LineChartData(
        titlesData: FlTitlesData(
          bottomTitles: AxisTitles(
            sideTitles: SideTitles(
              showTitles: true,
              reservedSize: 16,
              interval: 1,
              getTitlesWidget: createBottomTitleWidgets,
            ),
          ),
          rightTitles: AxisTitles(
            sideTitles: SideTitles(showTitles: false),
          ),
          topTitles: AxisTitles(
            sideTitles: SideTitles(showTitles: false),
          ),
          leftTitles: AxisTitles(
            sideTitles: SideTitles(
              showTitles: true,
              reservedSize: 32,
              interval: 32,
              getTitlesWidget: createLeftTitleWidgets,
            ),
          ),
        ),
        lineBarsData: [
          LineChartBarData(
            isCurved: true,
            color: Colors.green,
            barWidth: 8,
            isStrokeCapRound: true,
            dotData: FlDotData(show: true),
            belowBarData: BarAreaData(show: false),
            spots: spots,
          )
        ],
        minX: 0,
        maxX: _statisticsTables.isNotEmpty
            ? _statisticsTables.length.toDouble() - 1
            : 0,
        maxY: maxValue.toDouble(),
        minY: 0,
      ),
      swapAnimationDuration: const Duration(milliseconds: 150), // Optional
      swapAnimationCurve: Curves.linear, // Optional
    );
  }
}
