// ignore_for_file: unnecessary_brace_in_string_interps

import 'package:flutter/material.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';
import 'package:get/get.dart';
import 'package:sss/employeedetail.dart';
import 'package:sss/layout.dart';

import 'dialogs.dart';
import 'controllers.dart';
import 'controls.dart';
import 'viewmodels.dart';

typedef DepartmentSelectedCallback = void Function(
    DepartmentViewModel department);

class EmployeeModule extends StatelessWidget {
  final _filterController = TextEditingController();

  EmployeeModule({Key? key}) : super(key: key);

  @override
  Widget build(context) {
    _filterController.text = '';
    return LayoutUtils.isDesktopMode.value
        ? _desktopBuild(context)
        : _mobileBuild(context);
  }

  Widget _createParentBar(EmployeeController controller) {
    return Obx(() => Row(children: [
          controller.currentParentDepartment.value.parent != null
              ? SIconButton(
                  toolTip: '返回上一级',
                  icon: const Icon(Icons.keyboard_return),
                  onPressed: () async {
                    if (controller.currentParentDepartment.value.parent !=
                        null) {
                      var newParent = controller.currentParentDepartment.value
                          .parent as DepartmentViewModel;
                      await _loadDepartments(controller, newParent);
                    }
                  })
              : const SizedBox(width: 8),
          MouseRegion(
            cursor: SystemMouseCursors.click,
            child: GestureDetector(
                onTap: () {
                  _loadEmployees(
                      controller, controller.currentParentDepartment.value);
                },
                child: Text(controller.currentParentDepartment.value.name,
                    style: const TextStyle(
                        fontSize: 22, fontWeight: FontWeight.bold))),
          ),
        ]));
  }

  Widget _createDepartmentPanel(EmployeeController controller,
      DepartmentSelectedCallback onDepartmentSelected) {
    return SizedBox(
        width: 320,
        child: Column(
          children: [
            Container(
              padding: const EdgeInsets.fromLTRB(4, 8, 4, 4),
              alignment: Alignment.centerLeft,
              height: 48,
              child: _createParentBar(controller),
            ),
            const Divider(),
            Expanded(
                child: _createDepartmentList(controller, onDepartmentSelected))
          ],
        ));
  }

  Widget _createEmployeePanel(EmployeeController controller) {
    return Column(
      children: [
        SizedBox(
          height: 64,
          child: _createEmployeeOperationBar(controller),
        ),
        Expanded(
            child: Container(
          alignment: Alignment.topLeft,
          child: _createEmployeeList(controller),
        ))
      ],
    );
  }

  Widget _mobileBuild(BuildContext context) {
    final EmployeeController controller = Get.find<EmployeeController>();
    return Column(
      children: [
        Container(
          padding: const EdgeInsets.fromLTRB(4, 8, 4, 4),
          alignment: Alignment.centerLeft,
          height: 48,
          child: _createParentBar(controller),
        ),
        const Divider(),
        Expanded(
            child: _createDepartmentList(controller, (department) async {
          await _loadEmployees(controller, department);
          Get.toNamed('/employeelist');
        }))
      ],
    );
  }

  Widget _desktopBuild(BuildContext context) {
    final EmployeeController controller = Get.find<EmployeeController>();
    return Row(
      children: [
        _createDepartmentPanel(controller, (department) async {
          await _loadEmployees(controller, department);
        }),
        Container(width: 0.5, color: Colors.grey),
        Expanded(child: _createEmployeePanel(controller))
      ],
    );
  }

  Future _loadEmployees(
      EmployeeController controller, DepartmentViewModel department) async {
    EasyLoading.show(status: '正在加载员工信息...');
    try {
      await controller.loadEmployees(department);
      EasyLoading.dismiss();
    } catch (e) {
      EasyLoading.dismiss();
      Dialogs.showErrorDialog('加载员工信息失败:${e}');
    }
  }

  Future _refreshEmployees(EmployeeController controller) async {
    if (controller.selectedDepartment != null) {
      var department = controller.selectedDepartment as DepartmentViewModel;
      EasyLoading.show(status: '正在加载员工信息...');
      try {
        await controller.loadEmployees(department);
        EasyLoading.dismiss();
      } catch (e) {
        EasyLoading.dismiss();
        Dialogs.showErrorDialog('加载员工信息失败:${e}');
      }
    }
  }

  Future _loadDepartments(
      EmployeeController controller, DepartmentViewModel parent) async {
    EasyLoading.show(status: '正在加载部门列表...');
    try {
      await controller.loadDepartments(parent);
      EasyLoading.dismiss();
    } catch (e) {
      EasyLoading.dismiss();
      Dialogs.showErrorDialog('加载部门列表失败:${e}');
    }
  }

  Widget _createDepartmentItem(
      EmployeeController controller, DepartmentViewModel department) {
    return Row(mainAxisAlignment: MainAxisAlignment.center, children: [
      const Icon(Icons.group),
      const SizedBox(width: 2),
      Expanded(
          child: Tooltip(
              child: Text(department.name),
              message: '#' + department.data.id.toString())),
      SizedBox(
        width: 32,
        child: SIconButton(
            toolTip: '查看下级',
            icon: const Icon(Icons.arrow_forward_ios),
            onPressed: () async {
              _loadDepartments(controller, department);
            }),
      ),
    ]);
  }

  Widget _createDesktopDepartmentItem(
      EmployeeController controller, DepartmentViewModel department) {
    return MouseRegion(
      cursor: SystemMouseCursors.click,
      onHover: (event) => department.isHovering.value = true,
      onExit: (event) => department.isHovering.value = false,
      child: Obx(() => department.isHovering.value
          ? Container(
              decoration: BoxDecoration(
                border: Border.all(
                    color: const Color.fromARGB(255, 233, 234, 239), width: 2),
                color: const Color.fromARGB(150, 90, 181, 94),
                borderRadius: const BorderRadius.all(Radius.circular(10)),
              ),
              padding: const EdgeInsets.all(8),
              child: _createDepartmentItem(controller, department))
          : Container(
              padding: const EdgeInsets.all(8),
              child: _createDepartmentItem(controller, department))),
    );
  }

  Widget _createDepartmentList(EmployeeController controller,
      DepartmentSelectedCallback onDepartmentSelected) {
    return FutureBuilder(
        future: _loadDepartments(
            controller, controller.currentParentDepartment.value),
        builder: (context, snapshot) {
          return Obx(() => ListView.builder(
              controller: ScrollController(),
              padding: const EdgeInsets.all(4),
              itemCount: controller.departments.value.length,
              itemExtent: 56,
              itemBuilder: (BuildContext context, int index) {
                var department = controller.departments.value[index];
                return GestureDetector(
                    onTap: () {
                      onDepartmentSelected(department);
                    },
                    child: LayoutUtils.isDesktopMode.value
                        ? _createDesktopDepartmentItem(controller, department)
                        : Container(
                            padding: const EdgeInsets.all(8),
                            child:
                                _createDepartmentItem(controller, department)));
              }));
        });
  }

  Widget _createEmployeeOperationBar(EmployeeController controller) {
    return Row(
      children: [
        const SizedBox(
          width: 10,
        ),
        SizedBox(
            width: 256,
            height: 48,
            child: STextField(
                controller: _filterController,
                placeHolder: '输入员工名过滤',
                suffixIcon: SIconButton(
                  toolTip: '清除',
                  icon: const Icon(Icons.close),
                  onPressed: () {
                    _filterController.clear();
                    controller.filterKey = _filterController.text;
                    controller.filterEmployees();
                  },
                ))),
        const SizedBox(
          width: 10,
        ),
        SComButton(
            onPressed: () {
              controller.filterKey = _filterController.text;
              controller.filterEmployees();
            },
            icon: const Icon(Icons.search),
            label: const Text('过滤')),
        const SizedBox(
          width: 10,
        ),
        SComButton(
            onPressed: () => {_refreshEmployees(controller)},
            icon: const Icon(Icons.refresh),
            label: const Text('刷新')),
      ],
    );
  }

  Widget _createIsReviewerFlag() {
    return Container(
        width: 48,
        height: 20,
        decoration: BoxDecoration(
          border: Border.all(color: Colors.orangeAccent, width: 1),
          color: Colors.white,
          borderRadius: const BorderRadius.all(Radius.circular(4)),
        ),
        padding: const EdgeInsets.all(2),
        child: const Text('审核人员',
            style: TextStyle(color: Colors.orangeAccent, fontSize: 10)));
  }

  Widget _createIsStruggleFlag() {
    return Container(
        width: 48,
        height: 20,
        decoration: BoxDecoration(
          border: Border.all(color: Colors.blueAccent, width: 1),
          color: Colors.white,
          borderRadius: const BorderRadius.all(Radius.circular(4)),
        ),
        padding: const EdgeInsets.all(2),
        child: const Text('加班计划',
            style: TextStyle(color: Colors.blueAccent, fontSize: 10)));
  }

  Widget _createEmployeeItem(
      EmployeeController controller, EmployeeViewModel employee) {
    return Row(
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        Expanded(
            child: Row(children: [
          CircleAvatar(child: Text(employee.data.name[0])),
          const SizedBox(width: 8),
          Text(employee.data.name),
          const SizedBox(width: 4),
          Obx(() => employee.isReviewer.value
              ? _createIsReviewerFlag()
              : const SizedBox(width: 0)),
          const SizedBox(width: 4),
          Obx(() => employee.isStruggle.value
              ? _createIsStruggleFlag()
              : const SizedBox(width: 0)),
        ])),
        SizedBox(
            width: 40,
            child: Row(children: [
              SIconButton(
                  toolTip: '换部门',
                  onPressed: () {
                    _showChangeDepartmentDialog(controller, employee);
                  },
                  icon: const Icon(Icons.transfer_within_a_station))
            ])),
        const SizedBox(width: 10),
        SizedBox(
            width: 40,
            child: Row(children: [
              SIconButton(
                  toolTip: '删除',
                  onPressed: () =>
                      _showDeleteEmployeeDialog(controller, employee),
                  icon: const Icon(
                    Icons.delete,
                    color: Colors.red,
                  ))
            ]))
      ],
    );
  }

  Future _updateEmployeeDepartment(
      EmployeeController controller, EmployeeViewModel employee) async {
    EasyLoading.show(status: '正在提交员工信息...');
    try {
      await controller.updateEmployeeDepartment(employee);
      EasyLoading.dismiss();
    } catch (e) {
      EasyLoading.dismiss();
      Dialogs.showErrorDialog('更新员工信息失败: ${e}');
    }
  }

  void _showChangeDepartmentDialog(
      EmployeeController controller, EmployeeViewModel employee) {
    var tempController = EmployeeController();
    Dialogs.showChildDialog2(
        SizedBox(
            width: 320,
            height: 500,
            child: Column(
              children: [
                const Text('选择要迁入的部门', style: TextStyle(fontSize: 26)),
                const Divider(),
                const SizedBox(height: 16),
                Expanded(
                    child: _createDepartmentPanel(tempController,
                        (department) async {
                  if (employee.data.departmentId != department.data.id) {
                    //换部门
                    employee.data.departmentId = department.data.id;
                    await _updateEmployeeDepartment(controller, employee);
                    Dialogs.closeCurrentDialog();
                  } else {
                    Dialogs.closeCurrentDialog();
                  }
                }))
              ],
            )),
        actions: [
          SButton(
              onPressed: () => Dialogs.closeCurrentDialog(),
              child: const Text('取消'))
        ]);
  }

  Widget _createEmployeeList(EmployeeController controller) {
    return Obx(() => ListView.builder(
        controller: ScrollController(),
        padding: EdgeInsets.zero,
        itemCount: controller.filterMode.value
            ? controller.filteredEmployees.value.length
            : controller.employees.value.length,
        itemExtent: 64,
        itemBuilder: (BuildContext context, int index) {
          var employees = controller.filterMode.value
              ? controller.filteredEmployees.value
              : controller.employees.value;
          var employee = employees[index];
          return GestureDetector(
              onTap: () {
                Dialogs.showChildDialog2(SizedBox(
                  width: 350,
                  height: 300,
                  child: EmployeeDetailPage(employee: employee),
                ));
              },
              child: MouseRegion(
                cursor: SystemMouseCursors.click,
                onHover: (event) => employee.isHovering.value = true,
                onExit: (event) => employee.isHovering.value = false,
                child: Obx(
                  () => employee.isHovering.value
                      ? Container(
                          margin: const EdgeInsets.all(4),
                          padding: const EdgeInsets.all(8),
                          decoration: BoxDecoration(
                            border: Border.all(
                                color: const Color.fromARGB(255, 233, 234, 239),
                                width: 2),
                            color: const Color.fromARGB(255, 242, 242, 242),
                            borderRadius:
                                const BorderRadius.all(Radius.circular(2)),
                          ),
                          child: _createEmployeeItem(controller, employee))
                      : Container(
                          margin: const EdgeInsets.all(4),
                          padding: const EdgeInsets.all(8),
                          child: _createEmployeeItem(controller, employee)),
                ),
              ));
        }));
  }

  Future _deleteEmployee(
      EmployeeController ec, EmployeeViewModel employee) async {
    EasyLoading.show(status: '正在删除员工...');
    try {
      await ec.deleteEmployee(employee);
      EasyLoading.dismiss();
      Dialogs.closeCurrentDialog();
    } catch (e) {
      EasyLoading.dismiss();
      Dialogs.showErrorDialog('删除员工失败: $e');
    }
  }

  void _showDeleteEmployeeDialog(
      EmployeeController ec, EmployeeViewModel employee) {
    Dialogs.showConfirmDialog('确认删除此员工?', () async {
      await _deleteEmployee(ec, employee);
    });
  }
}
