import 'package:flutter/material.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';
import 'package:get/get.dart';
import 'package:sss/controllers.dart';
import 'package:sss/dialogs.dart';
import 'package:sss/viewmodels.dart';

class EmployeeDetailPage extends StatelessWidget {
  final EmployeeViewModel employee;

  const EmployeeDetailPage({Key? key, required this.employee})
      : super(key: key);

  @override
  Widget build(context) {
    EmployeeController controller = Get.find<EmployeeController>();
    return Column(
      children: [
        Row(
          children: [
            CircleAvatar(
              child: Text(employee.data.name[0]),
            ),
            const SizedBox(width: 4),
            Text(employee.data.name)
          ],
        ),
        const Divider(),
        const SizedBox(
          height: 12,
        ),
        Row(
          children: [
            SizedBox(
              width: 100,
              child: Row(children: const [
                Icon(Icons.title),
                SizedBox(width: 2),
                Text('职位')
              ]),
            ),
            Expanded(
                child: Text(
              employee.data.title,
              textAlign: TextAlign.end,
            )),
            const SizedBox(width: 8)
          ],
        ),
        const SizedBox(
          height: 20,
        ),
        Row(
          children: [
            SizedBox(
              width: 100,
              child: Row(children: const [
                Icon(Icons.work),
                SizedBox(width: 2),
                Text('班次')
              ]),
            ),
            Expanded(
                child: Text(
              employee.data.attendanceRules.title,
              textAlign: TextAlign.end,
            )),
            const SizedBox(width: 8)
          ],
        ),
        const SizedBox(
          height: 20,
        ),
        Row(
          children: [
            SizedBox(
              width: 100,
              child: Row(children: const [
                Icon(Icons.email),
                SizedBox(width: 2),
                Text('邮箱')
              ]),
            ),
            Expanded(
                child: Text(
              employee.data.email,
              textAlign: TextAlign.end,
            )),
            const SizedBox(width: 8)
          ],
        ),
        const SizedBox(
          height: 20,
        ),
        Row(
          children: [
            SizedBox(
              width: 100,
              child: Row(children: const [
                Icon(Icons.reviews),
                SizedBox(width: 2),
                Text('审核人员')
              ]),
            ),
            Expanded(
                child: Container(
              alignment: Alignment.centerRight,
              child: Obx(() => Switch(
                  value: employee.isReviewer.value,
                  onChanged: (value) {
                    employee.isReviewer.value = value;
                    _updateEmployeeIsReviewer(controller, employee);
                  })),
            ))
          ],
        ),
        const SizedBox(
          height: 8,
        ),
        Row(
          children: [
            SizedBox(
              width: 100,
              child: Row(children: const [
                Icon(Icons.hardware),
                SizedBox(width: 2),
                Text('是否加班')
              ]),
            ),
            Expanded(
                child: Container(
              alignment: Alignment.centerRight,
              child: Obx(() => Switch(
                  value: employee.isStruggle.value,
                  onChanged: (value) {
                    employee.isStruggle.value = value;
                    _updateEmployeeIsStruggle(controller, employee);
                  })),
            ))
          ],
        ),
      ],
    );
  }
}

Future _updateEmployeeIsReviewer(
    EmployeeController controller, EmployeeViewModel employee) async {
  EasyLoading.show(status: '正在提交员工信息...');
  try {
    await controller.updateEmployeeIsReviewer(employee);
    EasyLoading.dismiss();
  } catch (e) {
    EasyLoading.dismiss();
    Dialogs.showErrorDialog('更新员工信息失败: ${e}');
  }
}

Future _updateEmployeeIsStruggle(
    EmployeeController controller, EmployeeViewModel employee) async {
  EasyLoading.show(status: '正在提交员工信息...');
  try {
    await controller.updateEmployeeIsStruggle(employee);
    EasyLoading.dismiss();
  } catch (e) {
    EasyLoading.dismiss();
    Dialogs.showErrorDialog('更新员工信息失败: ${e}');
  }
}
