import 'package:flutter/material.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';
import 'package:get/get.dart';
import 'package:sss/viewmodels.dart';

import 'controllers.dart';
import 'controls.dart';
import 'dialogs.dart';
import 'employeedetail.dart';

class EmployeeListPage extends StatelessWidget {
  final _employeeFilterContrtoller = TextEditingController();

  EmployeeListPage({super.key});

  @override
  Widget build(BuildContext context) {
    var controller = Get.find<EmployeeController>();
    return Scaffold(
        resizeToAvoidBottomInset: false,
        appBar: _createAppBar(controller),
        body: Obx(() => _createEmployeeList(controller)));
  }

  AppBar _createAppBar(EmployeeController controller) {
    return AppBar(title: Text(controller.selectedDepartment!.name), actions: [
      PopupMenuButton(
        tooltip: '过滤',
        itemBuilder: ((c) {
          return [
            PopupMenuItem(
              child: const Text('按姓名过滤员工'),
              onTap: () => Future.delayed(const Duration(milliseconds: 10))
                  .then((value) => _showEmployeeFilter(controller)),
            ),
            PopupMenuItem(
              child: const Text('清空过滤器'),
              onTap: () => _clearFilter(controller),
            )
          ];
        }),
        icon: const Icon(
          Icons.filter,
          color: Colors.white,
        ),
      )
    ]);
  }

  void _showEmployeeFilter(EmployeeController controller) {
    Dialogs.showChildDialog2(
        SizedBox(
            width: 320,
            height: 40,
            child: STextField(
                placeHolder: '输入员工名关键字',
                controller: _employeeFilterContrtoller)),
        actions: [
          SButton(
              onPressed: () {
                controller.filterKey = _employeeFilterContrtoller.text;
                controller.filterEmployees();
                Dialogs.closeCurrentDialog();
              },
              child: const Text('过滤')),
          SButton(
              onPressed: () => {Dialogs.closeCurrentDialog()},
              child: const Text('取消')),
        ]);
  }

  void _clearFilter(EmployeeController controller) {
    _employeeFilterContrtoller.clear();
    controller.filterKey = _employeeFilterContrtoller.text;
    controller.filterEmployees();
  }

  Widget _createIsReviewerFlag() {
    return Container(
        width: 48,
        height: 20,
        decoration: BoxDecoration(
          border: Border.all(color: Colors.orangeAccent, width: 1),
          color: Colors.white,
          borderRadius: const BorderRadius.all(Radius.circular(4)),
        ),
        padding: const EdgeInsets.all(2),
        child: const Text('审核人员',
            style: TextStyle(color: Colors.orangeAccent, fontSize: 10)));
  }

  Widget _createIsStruggleFlag() {
    return Container(
        width: 48,
        height: 20,
        decoration: BoxDecoration(
          border: Border.all(color: Colors.blueAccent, width: 1),
          color: Colors.white,
          borderRadius: const BorderRadius.all(Radius.circular(4)),
        ),
        padding: const EdgeInsets.all(2),
        child: const Text('加班计划',
            style: TextStyle(color: Colors.blueAccent, fontSize: 10)));
  }

  Widget _createEmployeeItem(
      EmployeeController controller, EmployeeViewModel employee) {
    return Row(
      mainAxisAlignment: MainAxisAlignment.center,
      children: [
        Expanded(
            child: Row(children: [
          CircleAvatar(child: Text(employee.data.name[0])),
          const SizedBox(width: 8),
          Text(employee.data.name),
          const SizedBox(width: 4),
          Obx(() => employee.isReviewer.value
              ? _createIsReviewerFlag()
              : const SizedBox(width: 0)),
          const SizedBox(width: 4),
          Obx(() => employee.isStruggle.value
              ? _createIsStruggleFlag()
              : const SizedBox(width: 0)),
        ])),
        SizedBox(
            width: 40,
            child: Row(children: [
              SIconButton(
                  toolTip: '删除',
                  onPressed: () =>
                      _showDeleteEmployeeDialog(controller, employee),
                  icon: const Icon(
                    Icons.delete,
                    color: Colors.red,
                  ))
            ]))
      ],
    );
  }

  Widget _createEmployeeList(EmployeeController controller) {
    return ListView.builder(
        controller: ScrollController(),
        padding: EdgeInsets.zero,
        itemCount: controller.filterMode.value
            ? controller.filteredEmployees.value.length
            : controller.employees.value.length,
        itemExtent: 64,
        itemBuilder: (BuildContext context, int index) {
          var employees = controller.filterMode.value
              ? controller.filteredEmployees.value
              : controller.employees.value;
          var employee = employees[index];
          return GestureDetector(
              onTap: () {
                Dialogs.showChildDialog2(SizedBox(
                  width: 350,
                  height: 320,
                  child: EmployeeDetailPage(employee: employee),
                ));
              },
              child: MouseRegion(
                cursor: SystemMouseCursors.click,
                onHover: (event) => employee.isHovering.value = true,
                onExit: (event) => employee.isHovering.value = false,
                child: Container(
                    margin: const EdgeInsets.all(4),
                    padding: const EdgeInsets.all(8),
                    child: _createEmployeeItem(controller, employee)),
              ));
        });
  }

  Future _deleteEmployee(
      EmployeeController ec, EmployeeViewModel employee) async {
    EasyLoading.show(status: '正在删除员工...');
    try {
      await ec.deleteEmployee(employee);
      EasyLoading.dismiss();
      Dialogs.closeCurrentDialog();
    } catch (e) {
      EasyLoading.dismiss();
      Dialogs.showErrorDialog('删除员工失败: $e');
    }
  }

  void _showDeleteEmployeeDialog(
      EmployeeController ec, EmployeeViewModel employee) {
    Dialogs.showConfirmDialog('确认删除此员工?', () async {
      await _deleteEmployee(ec, employee);
    });
  }
}
