// ignore_for_file: unnecessary_brace_in_string_interps

import 'package:flutter/material.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';
import 'package:flutter_number_picker/flutter_number_picker.dart';
import 'package:get/get.dart';
import 'package:intl/intl.dart';
import 'package:sss/controllers.dart';
import 'package:sss/controls.dart';
import 'package:sss/dialogs.dart';
import 'package:sss/layout.dart';
import 'package:sss/session.dart';
import 'package:sss/viewmodels.dart';
import 'lunarconverter.dart';

class EmployeeStatisticsPage extends StatelessWidget {
  final RxBool _isDesktopMode = true.obs;
  final RxDouble _width = 0.0.obs;
  final RxDouble _height = 0.0.obs;

  EmployeeStatisticsPage({Key? key}) : super(key: key);

  @override
  Widget build(context) {
    _isDesktopMode.value = LayoutUtils.isDesktopMode.value;
    _width.value = LayoutUtils.width.value;
    _height.value = LayoutUtils.height.value;
    LayoutUtils.isDesktopMode.listen((value) {
      Future.delayed(const Duration(milliseconds: 10)).then((value) {
        _isDesktopMode.value = LayoutUtils.isDesktopMode.value;
      });
    });
    LayoutUtils.width.listen((value) {
      Future.delayed(const Duration(milliseconds: 10)).then((value) {
        _width.value = LayoutUtils.width.value;
      });
    });
    LayoutUtils.height.listen((value) {
      Future.delayed(const Duration(milliseconds: 10)).then((value) {
        _height.value = LayoutUtils.height.value;
      });
    });
    if (Get.arguments == null) {
      Get.toNamed('/');
      return const SizedBox();
    }
    var arguments = Get.arguments as Map;
    if (arguments["sc"] == null) {
      Get.toNamed('/');
      return const SizedBox();
    }
    var controller = arguments["sc"] as StatisticsController;
    var employeeStatistics = controller.selectedWorkingDayStatistics
        as WorkingDayStatisticsViewModel;
    return Scaffold(
        resizeToAvoidBottomInset: false,
        appBar: AppBar(
          title: Text(employeeStatistics.data.employee.name + ' 工时详情'),
          actions: [
            SIconButton(
              toolTip: '添加备注',
              onPressed: () =>
                  {_showAddCommentDialog(controller, employeeStatistics)},
              icon: const Icon(
                Icons.edit,
                size: 20,
                color: Colors.white,
              ),
            )
          ],
        ),
        body: Column(
          children: [
            const SizedBox(
              height: 4,
            ),
            Expanded(
              child: Obx(() => _isDesktopMode.value
                  ? _createDesktopView(controller, employeeStatistics)
                  : _createMobileView(controller, employeeStatistics)),
            )
          ],
        ));
  }

  Future _addComment(
      StatisticsController controller,
      WorkingDayStatisticsViewModel workingDayStatistics,
      String comment) async {
    EasyLoading.show(status: '正在更新备注...');
    try {
      await controller.updateWorkingDayStatisticsComment(
          workingDayStatistics.data.id, comment);
      workingDayStatistics.comment.value = comment;
      EasyLoading.dismiss();
      Dialogs.closeCurrentDialog();
    } catch (e) {
      EasyLoading.dismiss();
      Dialogs.showErrorDialog('更新备注失败: ${e.toString()}');
    }
  }

  void _showAddCommentDialog(StatisticsController controller,
      WorkingDayStatisticsViewModel workingDayStatistics) {
    var tec = TextEditingController();
    Dialogs.showChildDialog2(
        SizedBox(
          width: 320,
          height: 100,
          child: Column(
            children: [
              const Text('添加备注', style: TextStyle(fontSize: 26)),
              const Divider(),
              STextField(controller: tec, placeHolder: '输入备注信息'),
            ],
          ),
        ),
        actions: [
          SButton(
              onPressed: () =>
                  {_addComment(controller, workingDayStatistics, tec.text)},
              child: const Text('确定')),
          SButton(
              onPressed: () => {Dialogs.closeCurrentDialog()},
              child: const Text('取消')),
        ]);
  }

  Widget _createMobileView(StatisticsController controller,
      WorkingDayStatisticsViewModel employeeStatistics) {
    return Obx(() => ListView.builder(
        controller: ScrollController(),
        padding: EdgeInsets.zero,
        itemCount: controller.workingDayInfos.value.length,
        itemExtent: 128,
        itemBuilder: (BuildContext context, int index) {
          var workingDayInfo = controller.workingDayInfos.value[index];
          return Container(
              margin: const EdgeInsets.all(4),
              child: Column(
                children: [
                  Expanded(
                      child: _createWorkingDayView(controller, workingDayInfo)),
                  Container(
                    height: 0.5,
                    color: Colors.grey,
                  )
                ],
              ));
        }));
  }

  Widget _createDesktopView(StatisticsController controller,
      WorkingDayStatisticsViewModel employeeStatistics) {
    return _width.value > 1280
        ? _createCalendarView(controller, employeeStatistics)
        : FittedBox(
            fit: BoxFit.fitWidth,
            child: SizedBox(
                width: 1280,
                height: 1280 / (_width.value / _height.value),
                child: _createCalendarView(controller, employeeStatistics)));
  }

  Widget _createCalendarView(StatisticsController controller,
      WorkingDayStatisticsViewModel employeeStatistics) {
    return SCalendarView(
        start: employeeStatistics.data.start,
        end: employeeStatistics.data.end.add(const Duration(days: 1)),
        cellBuilder: (day) {
          var workingDayInfo = controller.workingDayInfos.value
              .firstWhereOrNull((element) =>
                  element.data.day.year == day.year &&
                  element.data.day.month == day.month &&
                  element.data.day.day == day.day);
          return _createWorkingDayView(controller, workingDayInfo);
        });
  }

  Widget _getStartTime(WorkingDayInfoViewModel workingDayInfo) {
    if (workingDayInfo.data.startResult == '') {
      return Row(children: [
        Container(
          width: 6,
          height: 6,
          decoration: const BoxDecoration(
              color: Colors.grey,
              borderRadius: BorderRadius.all(Radius.circular(5))),
        ),
        const SizedBox(width: 4),
        Text('上班 ' + DateFormat('HH:mm').format(workingDayInfo.data.start),
            style: const TextStyle(fontSize: 12))
      ]);
    }
    return Row(children: [
      Container(
        width: 6,
        height: 6,
        decoration: const BoxDecoration(
            color: Colors.red,
            borderRadius: BorderRadius.all(Radius.circular(5))),
      ),
      const SizedBox(width: 4),
      Tooltip(
          child: Text(
              '上班 ' + DateFormat('HH:mm').format(workingDayInfo.data.start),
              style: const TextStyle(color: Colors.red, fontSize: 12)),
          message: workingDayInfo.data.startResult,
          textStyle: const TextStyle(fontSize: 14, color: Colors.white))
    ]);
  }

  Widget _getEndTime(WorkingDayInfoViewModel workingDayInfo) {
    if (workingDayInfo.data.endResult == '') {
      return Row(children: [
        Container(
          width: 6,
          height: 6,
          decoration: const BoxDecoration(
              color: Colors.grey,
              borderRadius: BorderRadius.all(Radius.circular(5))),
        ),
        const SizedBox(width: 4),
        Text('下班 ' + DateFormat('HH:mm').format(workingDayInfo.data.end),
            style: const TextStyle(fontSize: 12))
      ]);
    }
    return Row(children: [
      Container(
        width: 6,
        height: 6,
        decoration: const BoxDecoration(
            color: Colors.red,
            borderRadius: BorderRadius.all(Radius.circular(5))),
      ),
      const SizedBox(width: 4),
      Tooltip(
          child: Text(
              '下班 ' + DateFormat('HH:mm').format(workingDayInfo.data.end),
              style: const TextStyle(color: Colors.red, fontSize: 12)),
          message: workingDayInfo.data.endResult,
          textStyle: const TextStyle(fontSize: 14, color: Colors.white))
    ]);
  }

  Widget _getWorkingTime(WorkingDayInfoViewModel workingDayInfo) {
    NumberFormat nf = NumberFormat('0.0');
    return Row(children: [
      Text('出勤工时: ' + nf.format(workingDayInfo.data.workingHours) + '小时',
          style: const TextStyle(fontSize: 10, color: Colors.grey))
    ]);
  }

  Widget _getStateText(WorkingDayInfoViewModel workingDayInfo) {
    if (workingDayInfo.adjustedObtainedUnits.value > 0) {
      return Stack(alignment: Alignment.center, children: [
        const Icon(
          Icons.brightness_low,
          color: Colors.green,
          size: 48,
        ),
        Text(
          workingDayInfo.adjustedObtainedUnits.value.toString(),
          textAlign: TextAlign.center,
          style: const TextStyle(color: Colors.green, fontSize: 12),
        )
      ]);
    } else if (workingDayInfo.data.isHoliday) {
      if (workingDayInfo.data.state == 5) {
        return Stack(alignment: Alignment.center, children: const [
          Icon(
            Icons.panorama_fish_eye,
            color: Colors.red,
            size: 48,
          ),
          Text(
            '加班',
            textAlign: TextAlign.center,
            style: TextStyle(color: Colors.red, fontSize: 12),
          )
        ]);
      }
      return Stack(alignment: Alignment.center, children: const [
        Icon(
          Icons.panorama_fish_eye,
          color: Colors.orange,
          size: 48,
        ),
        Text(
          '休息',
          style: TextStyle(color: Colors.orange, fontSize: 12),
        )
      ]);
    } else if (workingDayInfo.data.isWeekend) {
      if (workingDayInfo.data.state == 5) {
        return Stack(alignment: Alignment.center, children: const [
          Icon(
            Icons.panorama_fish_eye,
            color: Colors.red,
            size: 48,
          ),
          Text(
            '加班',
            style: TextStyle(color: Colors.red, fontSize: 12),
          )
        ]);
      }
      return Stack(alignment: Alignment.center, children: const [
        Icon(
          Icons.panorama_fish_eye,
          color: Colors.orange,
          size: 48,
        ),
        Text(
          '周末',
          style: TextStyle(color: Colors.orange, fontSize: 12),
        )
      ]);
    } else {
      switch (workingDayInfo.data.state) {
        case 0:
          return Stack(alignment: Alignment.center, children: const [
            Icon(
              Icons.panorama_fish_eye,
              color: Colors.black54,
              size: 48,
            ),
            Text(
              '出勤',
              style: TextStyle(color: Colors.black54, fontSize: 12),
            )
          ]);
        case 1:
          return Stack(alignment: Alignment.center, children: const [
            Icon(
              Icons.panorama_fish_eye,
              color: Colors.red,
              size: 48,
            ),
            Text(
              '请假',
              style: TextStyle(color: Colors.red, fontSize: 12),
            )
          ]);
        case 2:
          return Stack(alignment: Alignment.center, children: const [
            Icon(
              Icons.panorama_fish_eye,
              color: Colors.black54,
              size: 48,
            ),
            Text(
              '出差',
              style: TextStyle(color: Colors.black54, fontSize: 12),
            )
          ]);
        case 3:
          return Stack(alignment: Alignment.center, children: const [
            Icon(
              Icons.panorama_fish_eye,
              color: Colors.black54,
              size: 48,
            ),
            Text(
              '外出',
              style: TextStyle(color: Colors.black54, fontSize: 12),
            )
          ]);
        case 4:
          return Stack(alignment: Alignment.center, children: const [
            Icon(
              Icons.panorama_fish_eye,
              color: Colors.red,
              size: 48,
            ),
            Text(
              '缺卡',
              style: TextStyle(color: Colors.red, fontSize: 12),
            )
          ]);
        case 5:
          return Stack(alignment: Alignment.center, children: const [
            Icon(
              Icons.panorama_fish_eye,
              color: Colors.red,
              size: 48,
            ),
            Text(
              '加班',
              style: TextStyle(color: Colors.red, fontSize: 12),
            )
          ]);
        case 6:
          return Stack(alignment: Alignment.center, children: const [
            Icon(
              Icons.panorama_fish_eye,
              color: Colors.orange,
              size: 48,
            ),
            Text(
              '休息',
              style: TextStyle(color: Colors.orange, fontSize: 12),
            )
          ]);
        case 7:
          return Stack(alignment: Alignment.center, children: const [
            Icon(
              Icons.panorama_fish_eye,
              color: Colors.blue,
              size: 48,
            ),
            Text(
              '调班',
              style: TextStyle(color: Colors.blue, fontSize: 12),
            )
          ]);
      }
    }
    return const Text('',
        style: TextStyle(fontWeight: FontWeight.bold, fontSize: 12));
  }

  Widget _getStateView(WorkingDayInfoViewModel workingDayInfo) {
    return workingDayInfo.adjusted.value
        ? Stack(
            alignment: Alignment.topCenter,
            children: [
              Container(
                alignment: Alignment.center,
                width: 48,
                height: 48,
                child: _getStateText(workingDayInfo),
              ),
              Tooltip(
                message: workingDayInfo.adjustDescription.value,
                child: Container(
                    width: 48,
                    height: 48,
                    alignment: Alignment.bottomRight,
                    child: const Icon(
                      Icons.pending,
                      size: 20,
                    )),
              )
            ],
          )
        : Container(
            alignment: Alignment.center,
            width: 48,
            height: 48,
            child: _getStateText(workingDayInfo),
          );
  }

  Widget _createWorkingDayView(StatisticsController controller,
      WorkingDayInfoViewModel? workingDayInfo) {
    if (workingDayInfo == null) {
      return const SizedBox();
    } else {
      var employee = controller.selectedWorkingDayStatistics!.data.employee;
      var disableAdjust = (workingDayInfo.data.isHoliday &&
              !workingDayInfo.data.holidayAdjusted) ||
          (workingDayInfo.data.state == 0 &&
              (workingDayInfo.data.startResult == '' &&
                  workingDayInfo.data.endResult == ''));
      //如果原始数据达标，则可以调整，通常为调整为不达标。
      if (workingDayInfo.data.obtainedUnits > 0) {
        disableAdjust = false;
      }
      var day = workingDayInfo.data.day;
      var solar =
          Solar(solarYear: day.year, solarMonth: day.month, solarDay: day.day);
      var lunar = LunarSolarConverter.solarToLunar(solar).toShortString();
      return Container(
          padding: const EdgeInsets.all(8),
          alignment: Alignment.centerLeft,
          child: Column(
            mainAxisAlignment: MainAxisAlignment.start,
            children: [
              Row(
                mainAxisAlignment: MainAxisAlignment.center,
                children: [
                  Text(day.day.toString()),
                  const SizedBox(
                    width: 4,
                  ),
                  Text(
                    lunar,
                    style: const TextStyle(color: Colors.grey, fontSize: 10),
                  )
                ],
              ),
              const SizedBox(height: 4),
              Expanded(
                  child: ListView(
                children: [
                  Text('[${employee.attendanceRules.title}]',
                      textAlign: TextAlign.center,
                      style: const TextStyle(fontSize: 10)),
                  const SizedBox(
                    height: 4,
                  ),
                  Row(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      Expanded(
                        child: Column(
                          mainAxisAlignment: MainAxisAlignment.start,
                          crossAxisAlignment: CrossAxisAlignment.start,
                          children: [
                            _getWorkingTime(workingDayInfo),
                            const SizedBox(height: 6),
                            _getStartTime(workingDayInfo),
                            const SizedBox(height: 4),
                            _getEndTime(workingDayInfo),
                            const SizedBox(height: 4),
                          ],
                        ),
                      ),
                      disableAdjust
                          ? Container(
                              width: 50,
                              alignment: Alignment.topCenter,
                              child: Obx(() => _getStateView(workingDayInfo)))
                          : Container(
                              width: 50,
                              alignment: Alignment.topCenter,
                              child: GestureDetector(
                                onTap: () => {
                                  _showEditDayDialog(controller, workingDayInfo)
                                },
                                child: MouseRegion(
                                  cursor: SystemMouseCursors.click,
                                  child:
                                      Obx(() => _getStateView(workingDayInfo)),
                                ),
                              )),
                    ],
                  )
                ],
              ))
            ],
          ));
    }
  }

  void _adjustWorkingDayInfo(StatisticsController sc, int units, String desc,
      WorkingDayInfoViewModel workingDayInfo) {
    if (workingDayInfo.adjustedObtainedUnits.value != units && desc == '') {
      Dialogs.showErrorDialog('调整达标次数需输入调整原因。');
      return;
    }
    workingDayInfo.adjustedObtainedUnits.value = units;
    workingDayInfo.adjustDescription.value = desc;
    workingDayInfo.adjusted.value =
        (workingDayInfo.adjustedObtainedUnits.value !=
            workingDayInfo.data.obtainedUnits);
    Dialogs.closeCurrentDialog();
    _submitWorkingDayInfo(sc, workingDayInfo);
  }

  void _showEditDayDialog(
      StatisticsController sc, WorkingDayInfoViewModel workingDayInfo) {
    var obtainedUnits = RxInt(workingDayInfo.adjustedObtainedUnits.value);
    var adjustedDescription = TextEditingController(text: '');
    Dialogs.showChildDialog2(
        SizedBox(
            width: 320,
            height: 220,
            child: Column(
              mainAxisAlignment: MainAxisAlignment.start,
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                Container(
                    width: 320,
                    alignment: Alignment.center,
                    child:
                        const Text('调整工时信息', style: TextStyle(fontSize: 26))),
                const Divider(),
                const Text('调整次数'),
                const SizedBox(height: 4),
                SizedBox(
                    height: 48,
                    child: Obx(() => CustomNumberPicker(
                          valueTextStyle: const TextStyle(
                              fontFamily: 'Segoe UI', fontSize: 20),
                          initialValue: obtainedUnits.value,
                          maxValue: (workingDayInfo.data.isWeekend ||
                                  workingDayInfo.data.isHoliday)
                              ? 3
                              : 1,
                          minValue: 0,
                          step: (workingDayInfo.data.isWeekend ||
                                  workingDayInfo.data.isHoliday)
                              ? 3
                              : 1,
                          shape: OutlineInputBorder(
                              //未选中时候的颜色
                              borderRadius: BorderRadius.circular(4.0),
                              borderSide: const BorderSide(
                                  color: Color.fromARGB(255, 225, 227, 233),
                                  width: 2)),
                          onValue: (value) {
                            obtainedUnits.value = value as int;
                          },
                        ))),
                const SizedBox(height: 10),
                Column(
                    mainAxisAlignment: MainAxisAlignment.start,
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      const Text('调整原因'),
                      const SizedBox(height: 4),
                      STextField(
                        controller: adjustedDescription,
                      ),
                    ]),
              ],
            )),
        actions: [
          SButton(
              onPressed: () => {
                    _adjustWorkingDayInfo(sc, obtainedUnits.value,
                        adjustedDescription.text, workingDayInfo)
                  },
              child: const Text('确定')),
          SButton(
              onPressed: () => {Dialogs.closeCurrentDialog()},
              child: const Text('取消'))
        ]);
  }

  Future _submitWorkingDayInfo(StatisticsController controller,
      WorkingDayInfoViewModel workingDayInfo) async {
    EasyLoading.show(status: '正在提交修改信息...');
    try {
      var adjustedUnit = workingDayInfo.adjustedObtainedUnits.value -
          workingDayInfo.data.adjustedObtainedUnits;
      if (adjustedUnit != 0) {
        //update working day
        workingDayInfo.data.adjustedObtainedUnits += adjustedUnit;
        workingDayInfo.data.adjusted = workingDayInfo.adjusted.value;
        workingDayInfo.data.adjustDescription =
            workingDayInfo.adjustDescription.value;
        await controller.updateWorkingDayInfo(workingDayInfo.data);
      }
      //Update statistcs
      var adjustedReachedCount = 0;
      for (var element in controller.workingDayInfos.value) {
        adjustedReachedCount += element.adjustedObtainedUnits.value;
      }
      var adjuested = controller.workingDayInfos.value
          .any((element) => element.adjusted.value);
      var adjustDescription = '';
      for (var element in controller.workingDayInfos.value) {
        if (element.data.adjustDescription != '') {
          if (adjustDescription == '') {
            adjustDescription += '[' +
                DateFormat('yyyy-MM-dd').format(element.data.day) +
                '] ' +
                element.adjustDescription.value;
          } else {
            adjustDescription += '\r\n[' +
                DateFormat('yyyy-MM-dd').format(element.data.day) +
                '] ' +
                element.adjustDescription.value;
          }
        }
      }
      var workingDayStatistics = controller.selectedWorkingDayStatistics!;
      workingDayStatistics.adjustedReachedCount.value = adjustedReachedCount;
      workingDayStatistics.data.adjustedReachedCount = adjustedReachedCount;
      workingDayStatistics.adjusted.value = adjuested;
      workingDayStatistics.data.adjusted = adjuested;
      workingDayStatistics.adjustDescription.value = adjustDescription;
      workingDayStatistics.data.adjustDescription = adjustDescription;
      workingDayStatistics.data.adjustedBy = Session.loginEmployee != null
          ? Session.loginEmployee!.name
          : Session.currentAccountName;
      await controller.updateWorkingDayStatistics(workingDayStatistics.data);
      EasyLoading.dismiss();
    } catch (e) {
      EasyLoading.dismiss();
      Dialogs.showErrorDialog('提交修改信息失败: ' + e.toString());
    }
  }
}
