// ignore_for_file: unnecessary_brace_in_string_interps

import 'package:get/get.dart';
import 'package:intl/intl.dart';

class AttendanceRules {
  String title;
  DateTime start;
  DateTime end;

  AttendanceRules(this.start, this.end, this.title);

  AttendanceRules.fromJosn(Map<String, dynamic> json)
      : title = json['Title'].toString(),
        start = DateTime.parse(json['Start']),
        end = DateTime.parse(json['End']);

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['Title'] = title;
    data['Start'] = DateFormat('yyyy-MM-dd HH:mm:ss').format(start);
    data['End'] = DateFormat('yyyy-MM-dd HH:mm:ss').format(end);
    return data;
  }
}

class Department {
  int id;
  String name;
  int parentId;
  Department(this.id, this.name, this.parentId);

  Department.fromJosn(Map<String, dynamic> json)
      : id = json['Id'] as int,
        name = json['Name'].toString(),
        parentId = json['ParentId'] as int;

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['Id'] = id;
    data['Name'] = name;
    return data;
  }
}

class Employee {
  String id;
  String name;
  String title;
  int departmentId;
  String email;
  AttendanceRules attendanceRules;
  bool isReviewer;
  bool isStruggle;
  Employee(this.id, this.name, this.title, this.departmentId, this.email,
      this.attendanceRules, this.isReviewer, this.isStruggle);

  Employee.fromJosn(Map<String, dynamic> json)
      : id = json['Id'].toString(),
        name = json['Name'].toString(),
        title = json['Title'].toString(),
        departmentId = json['DepartmentId'] as int,
        email = json['Email'].toString(),
        attendanceRules = AttendanceRules.fromJosn(
            json['AttendanceRules'] as Map<String, dynamic>),
        isReviewer = json['IsReviewer'] as bool,
        isStruggle = json['IsStruggle'] as bool;

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['Id'] = id;
    data['Name'] = name;
    data['Title'] = title;
    data['DepartmentId'] = departmentId;
    data['Email'] = email;
    data['AttendanceRules'] = attendanceRules.toJson();
    data['IsReviewer'] = isReviewer;
    data['IsStruggle'] = isStruggle;
    return data;
  }
}

class WorkingDay {
  int dayOfWeek;
  RxBool canStruggle;
  RxBool isStruggle;
  DateTime start;
  DateTime end;
  WorkingDay(
      this.dayOfWeek, this.canStruggle, this.isStruggle, this.start, this.end);
}

class ValidateResult {
  bool ok;
  String errorMessage;
  ValidateResult(this.ok, {this.errorMessage = ''});
}

class WorkingDayInfo {
  String id;
  String employeeId;
  DateTime day;
  DateTime start;
  String startResult;
  DateTime end;
  String endResult;
  double requiredWorkingHours;
  double workingHours;
  DateTime adjustedDay;
  bool isHoliday;
  bool holidayAdjusted;
  String holidayName;
  bool isWeekend;
  int state;
  int obtainedUnits;
  int adjustedObtainedUnits;
  String adjustDescription;
  bool adjusted;

  WorkingDayInfo(
      this.id,
      this.employeeId,
      this.day,
      this.start,
      this.startResult,
      this.end,
      this.endResult,
      this.requiredWorkingHours,
      this.workingHours,
      this.adjustedDay,
      this.isHoliday,
      this.holidayAdjusted,
      this.holidayName,
      this.isWeekend,
      this.state,
      this.obtainedUnits,
      this.adjustedObtainedUnits,
      this.adjustDescription,
      this.adjusted);

  WorkingDayInfo.fromJosn(Map<String, dynamic> json)
      : id = json['Id'].toString(),
        employeeId = json['EmployeeId'].toString(),
        day = DateTime.parse(json['Day'].toString()),
        start = DateTime.parse(json['Start'].toString()),
        startResult = json['StartResult'].toString(),
        end = DateTime.parse(json['End'].toString()),
        endResult = json['EndResult'].toString(),
        requiredWorkingHours = (json['RequiredWorkingHours'] as num).toDouble(),
        workingHours = (json['WorkingHours'] as num).toDouble(),
        adjustedDay = DateTime.parse(json['AdjustedDay'].toString()),
        isHoliday = json['IsHoliday'] as bool,
        holidayAdjusted = json['HolidayAdjusted'] as bool,
        holidayName = json['HolidayName'].toString(),
        isWeekend = json['IsWeekend'] as bool,
        state = json['State'] as int,
        obtainedUnits = json['ObtainedUnits'] as int,
        adjustedObtainedUnits = json['AdjustedObtainedUnits'] as int,
        adjustDescription = json['AdjustDescription'].toString(),
        adjusted = json['Adjusted'] as bool;

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['Id'] = id;
    data['EmployeeId'] = employeeId;
    data['Day'] = DateFormat('yyyy-MM-dd HH:mm:ss').format(day);
    data['Start'] = DateFormat('yyyy-MM-dd HH:mm:ss').format(start);
    data['StartResult'] = startResult;
    data['End'] = DateFormat('yyyy-MM-dd HH:mm:ss').format(end);
    data['EndResult'] = endResult;
    data['RequiredWorkingHours'] = requiredWorkingHours;
    data['WorkingHours'] = workingHours;
    data['AdjustedDay'] = DateFormat('yyyy-MM-dd HH:mm:ss').format(adjustedDay);
    data['IsHoliday'] = isHoliday;
    data['HolidayAdjusted'] = holidayAdjusted;
    data['HolidayName'] = holidayName;
    data['IsWeekend'] = isWeekend;
    data['State'] = state;
    data['ObtainedUnits'] = obtainedUnits;
    data['AdjustedObtainedUnits'] = adjustedObtainedUnits;
    data['AdjustDescription'] = adjustDescription;
    data['Adjusted'] = adjusted;
    return data;
  }
}

class WorkingDayStatistics {
  String id;
  Employee employee;
  int departmentId;
  String departmentName;
  DateTime start;
  DateTime end;
  int requiredReachedCount;
  int reachedCount;
  int adjustedReachedCount;
  bool adjusted;
  String adjustDescription;
  String adjustedBy;
  String comment;
  String reviewer;

  WorkingDayStatistics(
      this.id,
      this.employee,
      this.departmentId,
      this.departmentName,
      this.start,
      this.end,
      this.requiredReachedCount,
      this.reachedCount,
      this.adjustedReachedCount,
      this.adjusted,
      this.adjustDescription,
      this.adjustedBy,
      this.comment,
      this.reviewer);

  WorkingDayStatistics.fromJosn(Map<String, dynamic> json)
      : id = json['Id'].toString(),
        employee = Employee.fromJosn(json['Employee'] as Map<String, dynamic>),
        departmentId = int.parse(json['DepartmentId'].toString()),
        departmentName = json['DepartmentName'].toString(),
        start = DateTime.parse(json['Start'].toString()),
        end = DateTime.parse(json['End'].toString()),
        requiredReachedCount = json['RequiredReachedCount'] as int,
        reachedCount = json['ReachedCount'] as int,
        adjustedReachedCount = json['AdjustedReachedCount'] as int,
        adjusted = json['Adjusted'] as bool,
        adjustDescription = json['AdjustDescription'].toString(),
        adjustedBy = json['AdjustedBy'].toString(),
        comment = json['Comment'].toString(),
        reviewer = json['Reviewer'].toString();

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['Id'] = id;
    data['Employee'] = employee;
    data['DepartmentId'] = departmentId;
    data['DepartmentName'] = departmentName;
    data['Start'] = DateFormat('yyyy-MM-dd HH:mm:ss').format(start);
    data['End'] = DateFormat('yyyy-MM-dd HH:mm:ss').format(end);
    data['RequiredReachedCount'] = requiredReachedCount;
    data['ReachedCount'] = reachedCount;
    data['AdjustedReachedCount'] = adjustedReachedCount;
    data['Adjusted'] = adjusted;
    data['AdjustDescription'] = adjustDescription;
    data['AdjustedBy'] = adjustedBy;
    data['Comment'] = comment;
    data['Reviewer'] = reviewer;
    return data;
  }
}

class StatisticsTable {
  String id;
  String title;
  List<String> workingDayStatisticsIds;
  DateTime createTime;

  StatisticsTable(
      this.id, this.title, this.workingDayStatisticsIds, this.createTime);

  StatisticsTable.fromJosn(Map<String, dynamic> json)
      : id = json['Id'].toString(),
        title = json['Title'].toString(),
        workingDayStatisticsIds =
            (json['WorkingDayStatisticsIds'] as List<dynamic>)
                .map((e) => e.toString())
                .toList(),
        createTime = DateTime.parse(json['CreateTime'].toString())
            .add(const Duration(hours: 8));

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['Id'] = id;
    data['Title'] = title;
    data['WorkingDayStatisticsIds'] = workingDayStatisticsIds;
    data['CreateTime'] = DateFormat('yyyy-MM-dd HH:mm:ss').format(createTime);
    return data;
  }
}

class ViewCode {
  String id;
  String code;
  String receiver;
  List<String> workingDayStatisticsIds;
  DateTime expiredTime;

  ViewCode(this.id, this.code, this.receiver, this.workingDayStatisticsIds,
      this.expiredTime);

  ViewCode.fromJosn(Map<String, dynamic> json)
      : id = json['Id'].toString(),
        code = json['Code'].toString(),
        receiver = json['Receiver'].toString(),
        workingDayStatisticsIds =
            (json['WorkingDayStatisticsIds'] as List<dynamic>)
                .map((e) => e.toString())
                .toList(),
        expiredTime = DateTime.parse(json['ExpiredTime'].toString());

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['Id'] = id;
    data['Code'] = code;
    data['Receiver'] = receiver;
    data['WorkingDayStatisticsIds'] = workingDayStatisticsIds;
    data['ExpiredTime'] = DateFormat('yyyy-MM-dd HH:mm:ss').format(expiredTime);
    return data;
  }
}

class EmailSetting {
  String id;
  String host;
  String port;
  String sender;
  String senderPassword;

  EmailSetting(this.id, this.host, this.port, this.sender, this.senderPassword);

  EmailSetting.fromJosn(Map<String, dynamic> json)
      : id = json['Id'].toString(),
        host = json['Host'].toString(),
        port = json['Port'].toString(),
        sender = json['Sender'].toString(),
        senderPassword = json['SenderPassword'].toString();

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['Id'] = id;
    data['Host'] = host;
    data['Port'] = port;
    data['Sender'] = sender;
    data['SenderPassword'] = senderPassword;
    return data;
  }
}

class HolidayInfo {
  String id;
  String title;
  DateTime holiday;
  bool isAdjusted;
  bool isNationalLegal;
  DateTime adjustedDay;

  HolidayInfo(this.id, this.title, this.holiday, this.isAdjusted,
      this.isNationalLegal, this.adjustedDay);

  HolidayInfo.fromJosn(Map<String, dynamic> json)
      : id = json['Id'].toString(),
        title = json['Title'].toString(),
        holiday = DateTime.parse(json['Holiday'].toString()),
        isAdjusted = json['IsAdjusted'] as bool,
        isNationalLegal = json['IsNationalLegal'] == null
            ? false
            : json['IsNationalLegal'] as bool,
        adjustedDay = DateTime.parse(json['AdjustedDay'].toString());

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['Id'] = id;
    data['Title'] = title;
    data['Holiday'] = DateFormat('yyyy-MM-dd HH:mm:ss').format(holiday);
    data['IsAdjusted'] = isAdjusted;
    data['IsNationalLegal'] = isNationalLegal;
    data['AdjustedDay'] = DateFormat('yyyy-MM-dd HH:mm:ss').format(adjustedDay);
    return data;
  }
}

class DomainSetting {
  String id;
  String domain;

  DomainSetting(this.id, this.domain);

  DomainSetting.fromJosn(Map<String, dynamic> json)
      : id = json['Id'].toString(),
        domain = json['Domain'].toString();

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['Id'] = id;
    data['Domain'] = domain;
    return data;
  }
}

class MessageTemplate {
  String id;
  String name;
  String content;

  MessageTemplate(this.id, this.name, this.content);

  MessageTemplate.fromJosn(Map<String, dynamic> json)
      : id = json['Id'].toString(),
        name = json['Name'].toString(),
        content = json['Content'].toString();

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['Id'] = id;
    data['Name'] = name;
    data['Content'] = content;
    return data;
  }
}

class LeaveQuota {
  String id;
  String leaveType;
  int unit;
  int quota;
  int used;

  LeaveQuota(this.id, this.leaveType, this.unit, this.quota, this.used);

  LeaveQuota.fromJosn(Map<String, dynamic> json)
      : id = json['Id'].toString(),
        leaveType = json['LeaveType'].toString(),
        unit = (json['Unit'] as num).toInt(),
        quota = (json['Quota'] as num).toInt(),
        used = (json['Used'] as num).toInt();

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['Id'] = id;
    data['LeaveType'] = leaveType;
    data['Unit'] = unit;
    data['Quota'] = quota;
    data['Used'] = used;
    return data;
  }
}
