import 'package:flutter/material.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';
import 'package:get/get.dart';
import 'package:intl/intl.dart';
import 'package:sss/controllers.dart';
import 'package:sss/entities.dart';
import 'package:sss/layout.dart';
import 'package:uuid/uuid.dart';

import 'controls.dart';
import 'dialogs.dart';
import 'lunarconverter.dart';

class HolidayPage extends StatelessWidget {
  const HolidayPage({Key? key}) : super(key: key);

  @override
  Widget build(context) {
    HolidayController controller = Get.find<HolidayController>();
    return Row(
      children: [
        Obx(() => LayoutUtils.isDesktopMode.value
            ? _createDescriptionPanel()
            : const SizedBox()),
        Container(
          width: 0.5,
          color: Colors.grey,
        ),
        Expanded(
            child: Column(
          children: [
            SizedBox(
              height: 48,
              child: _createMonthPanel(controller),
            ),
            Expanded(
                child: Obx(() => LayoutUtils.isDesktopMode.value
                    ? _createCalendarView(controller)
                    : Column(
                        children: [
                          Container(height: 0.5, color: Colors.grey),
                          Expanded(child: _createListView(controller))
                        ],
                      )))
          ],
        ))
      ],
    );
  }

  Widget _createDescriptionPanel() {
    return Container(
      color: Colors.white,
      padding: const EdgeInsets.all(8),
      width: 200,
      child: Column(crossAxisAlignment: CrossAxisAlignment.start, children: [
        const Text(
          '说明',
          textAlign: TextAlign.left,
          style: TextStyle(color: Colors.grey),
        ),
        const SizedBox(
          height: 16,
        ),
        Row(children: [
          Container(
            width: 14,
            height: 14,
            decoration: const BoxDecoration(
                color: Colors.green,
                borderRadius: BorderRadius.all(Radius.circular(10))),
          ),
          const SizedBox(width: 8),
          const Text('节假日'),
        ]),
        const SizedBox(height: 10),
        Row(children: [
          Container(
            width: 14,
            height: 14,
            decoration: const BoxDecoration(
                color: Colors.red,
                borderRadius: BorderRadius.all(Radius.circular(10))),
          ),
          const SizedBox(width: 8),
          const Text('调班日'),
        ])
      ]),
    );
  }

  Widget _createMonthPanel(HolidayController controller) {
    return Obx(() => Container(
        color: Colors.white,
        child: Row(
          children: [
            const SizedBox(
              width: 8,
            ),
            SIconButton(
                toolTip: '上个月',
                onPressed: () {
                  controller.moveToPreviewMonth();
                },
                icon: const Icon(Icons.arrow_back_ios_new)),
            const SizedBox(
              width: 8,
            ),
            SIconButton(
                toolTip: '下个月',
                onPressed: () {
                  controller.moveToNextMonth();
                },
                icon: const Icon(Icons.arrow_forward_ios)),
            const SizedBox(
              width: 8,
            ),
            Text(
              '${controller.currentStart.value.year}年${controller.currentStart.value.month}月',
              style: const TextStyle(fontSize: 24),
            )
          ],
        )));
  }

  Widget _createHolidayView(HolidayController controller, HolidayInfo holiday) {
    return Column(
      children: [
        SizedBox(
            height: 20,
            child: Row(
              children: [
                Container(
                  width: 10,
                  height: 10,
                  decoration: const BoxDecoration(
                      color: Colors.green,
                      borderRadius: BorderRadius.all(Radius.circular(5))),
                ),
                const SizedBox(width: 4),
                Text(
                  holiday.title,
                  textAlign: TextAlign.left,
                  style: const TextStyle(color: Colors.green),
                ),
                Expanded(
                    child: Container(
                  alignment: Alignment.centerRight,
                  child: GestureDetector(
                    onTap: () =>
                        {_showDeleteHolidayDialog(controller, holiday)},
                    child: const MouseRegion(
                        cursor: SystemMouseCursors.click,
                        child: Icon(
                          Icons.delete,
                          size: 20,
                          color: Colors.red,
                        )),
                  ),
                ))
              ],
            )),
        LayoutUtils.isDesktopMode.value ? const Divider() : const SizedBox(),
        Expanded(
            child: ListView(
          itemExtent: 20,
          children: [
            Row(
              children: [
                SizedBox(
                  width: 80,
                  child: Row(children: const [
                    Icon(
                      Icons.hardware,
                      size: 12,
                    ),
                    SizedBox(width: 2),
                    Text(
                      '调班日期',
                      style: TextStyle(fontSize: 12),
                    )
                  ]),
                ),
                Expanded(
                    child: Text(
                  DateFormat('yyyy-MM-dd').format(holiday.adjustedDay),
                  textAlign: TextAlign.end,
                  style: const TextStyle(fontSize: 12),
                )),
                const SizedBox(width: 8)
              ],
            ),
            Row(
              children: [
                SizedBox(
                  width: 80,
                  child: Row(children: const [
                    Icon(
                      Icons.account_balance,
                      size: 12,
                    ),
                    SizedBox(width: 2),
                    Text(
                      '法定节假日',
                      style: TextStyle(fontSize: 12),
                    )
                  ]),
                ),
                Expanded(
                    child: Container(
                  alignment: Alignment.centerRight,
                  child: holiday.isNationalLegal
                      ? const Icon(
                          Icons.check,
                          size: 12,
                        )
                      : const Icon(Icons.clear, size: 12),
                )),
                const SizedBox(width: 8)
              ],
            ),
          ],
        ))
      ],
    );
  }

  Widget _createAdjustedDayView(
      HolidayController controller, HolidayInfo adjustedDay) {
    return Column(
      children: [
        SizedBox(
            height: 20,
            child: Row(
              children: [
                Container(
                  width: 10,
                  height: 10,
                  decoration: const BoxDecoration(
                      color: Colors.red,
                      borderRadius: BorderRadius.all(Radius.circular(5))),
                ),
                const SizedBox(width: 4),
                Text(
                  adjustedDay.title,
                  textAlign: TextAlign.left,
                  style: const TextStyle(color: Colors.red),
                ),
                Expanded(child: Container())
              ],
            )),
        LayoutUtils.isDesktopMode.value ? const Divider() : const SizedBox(),
        Expanded(
            child: ListView(
          itemExtent: 20,
          children: [
            Row(
              children: [
                SizedBox(
                  width: 80,
                  child: Row(children: const [
                    Icon(
                      Icons.free_breakfast,
                      size: 12,
                    ),
                    SizedBox(width: 2),
                    Text(
                      '调班假日',
                      style: TextStyle(fontSize: 12),
                    )
                  ]),
                ),
                Expanded(
                    child: Text(
                  DateFormat('yyyy-MM-dd').format(adjustedDay.holiday),
                  textAlign: TextAlign.end,
                  style: const TextStyle(fontSize: 12),
                )),
                const SizedBox(width: 8)
              ],
            ),
          ],
        ))
      ],
    );
  }

  Widget _createDayView(HolidayController controller, DateTime day) {
    var holiday = controller.holidays.value.firstWhereOrNull(
      (element) => DateUtils.isSameDay(day, element.holiday),
    );
    var adjuestedDay = controller.holidays.value.firstWhereOrNull(
      (element) => DateUtils.isSameDay(day, element.adjustedDay),
    );
    var solar =
        Solar(solarYear: day.year, solarMonth: day.month, solarDay: day.day);
    var lunar = LunarSolarConverter.solarToLunar(solar).toShortString();
    return Container(
      padding: const EdgeInsets.all(4),
      alignment: Alignment.topCenter,
      child: Column(mainAxisAlignment: MainAxisAlignment.start, children: [
        Row(
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            Text(day.day.toString()),
            const SizedBox(
              width: 4,
            ),
            Text(
              lunar,
              style: const TextStyle(color: Colors.grey, fontSize: 10),
            )
          ],
        ),
        const SizedBox(height: 4),
        holiday == null
            ? (adjuestedDay == null
                ? Expanded(
                    child: MouseRegion(
                        cursor: SystemMouseCursors.click,
                        child: GestureDetector(
                            behavior: HitTestBehavior.opaque,
                            onTap: () {
                              _showCreateHolidyDialog(controller, day);
                            },
                            child: Container())))
                : Expanded(
                    child: _createAdjustedDayView(controller, adjuestedDay)))
            : Expanded(child: _createHolidayView(controller, holiday)),
      ]),
    );
  }

  Widget _createListView(HolidayController controller) {
    return FutureBuilder(
        future: _loadHolidays(controller),
        builder: (context, snapshot) {
          if (snapshot.connectionState == ConnectionState.done) {
            return Obx(() => ListView.builder(
                controller: ScrollController(),
                itemCount: DateUtils.getDaysInMonth(
                    controller.currentStart.value.year,
                    controller.currentStart.value.month),
                itemExtent: 96,
                itemBuilder: (BuildContext context, int index) {
                  var day = DateUtils.addDaysToDate(
                      controller.currentStart.value, index);
                  return Obx(() => Column(
                        children: [
                          Expanded(
                              child: Container(
                            padding: const EdgeInsets.fromLTRB(10, 0, 10, 0),
                            child: _createDayView(controller, day),
                          )),
                          Container(height: 0.5, color: Colors.grey)
                        ],
                      ));
                }));
          } else {
            return const SizedBox();
          }
        });
  }

  Widget _createCalendarView(HolidayController controller) {
    return FutureBuilder(
        future: _loadHolidays(controller),
        builder: (context, snapshot) {
          if (snapshot.connectionState == ConnectionState.done) {
            return Obx(() => SCalendarView(
                start: controller.currentStart.value,
                end: controller.currentEnd.value,
                cellBuilder: (day) {
                  return Obx(() => _createDayView(controller, day));
                }));
          } else {
            return const SizedBox();
          }
        });
  }

  Future _deleteHoliday(HolidayController hc, HolidayInfo holiday) async {
    EasyLoading.show(status: '正在删除节假日信息...');
    try {
      await hc.deleteHoliday(holiday);
      EasyLoading.dismiss();
      Dialogs.closeCurrentDialog();
    } catch (e) {
      EasyLoading.dismiss();
      Dialogs.showErrorDialog('删除节假日失败: $e');
    }
  }

  void _showDeleteHolidayDialog(HolidayController hc, HolidayInfo holiday) {
    Dialogs.showConfirmDialog('确认删除该节假日?', () {
      _deleteHoliday(hc, holiday);
    });
  }

  Future _addHoliday(HolidayController hc, String title, DateTime holiday,
      bool adjust, bool isNationalLegal, DateTime adjustedDay) async {
    if (title == '') {
      Dialogs.showErrorDialog('请输入节假日名。');
      return;
    }
    if (adjust && holiday == adjustedDay) {
      Dialogs.showErrorDialog('节假日和调整日不能为同一天。');
      return;
    }
    EasyLoading.show(status: '正在添加节假日信息...');
    try {
      var holidayInfo = HolidayInfo(const Uuid().v1(), title, holiday, adjust,
          isNationalLegal, adjustedDay);
      await hc.addHoliday(holidayInfo);
      EasyLoading.dismiss();
      Dialogs.closeCurrentDialog();
    } catch (e) {
      EasyLoading.dismiss();
      Dialogs.showErrorDialog('添加节假日失败: $e');
    }
  }

  Widget _createHolidayInfoView(
      TextEditingController titleController,
      PageController pageController,
      DateTime day,
      Rx<DateTime> adjustedDay,
      RxBool isAdjusted,
      RxBool isIsNationalLegal) {
    return Column(
      mainAxisAlignment: MainAxisAlignment.start,
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        const SizedBox(height: 18),
        Row(
          children: [
            const SizedBox(width: 100, child: Text('节假日')),
            Expanded(
                child: Container(
                    alignment: Alignment.centerRight,
                    child: Text(DateFormat('yyyy-MM-dd').format(day)))),
          ],
        ),
        const SizedBox(height: 24),
        const Text('节假日名'),
        const SizedBox(height: 4),
        STextField(controller: titleController),
        const SizedBox(
          height: 18,
        ),
        Row(
          children: [
            const SizedBox(width: 100, child: Text('是否调班')),
            Expanded(
                child: Container(
              alignment: Alignment.centerRight,
              child: Row(
                mainAxisAlignment: MainAxisAlignment.end,
                children: [
                  Obx(() => Container(
                      alignment: Alignment.centerRight,
                      child: isAdjusted.value
                          ? Text(DateFormat('yyyy-MM-dd')
                              .format(adjustedDay.value))
                          : const SizedBox())),
                  const SizedBox(
                    width: 8,
                  ),
                  Obx(() => Switch(
                      value: isAdjusted.value,
                      onChanged: ((value) {
                        isAdjusted.value = value;
                        if (value) {
                          pageController.animateToPage(1,
                              duration: const Duration(milliseconds: 200),
                              curve: Curves.linear);
                        }
                      })))
                ],
              ),
            )),
          ],
        ),
        const SizedBox(
          height: 18,
        ),
        Row(
          children: [
            const SizedBox(width: 100, child: Text('法定节假日')),
            Expanded(
                child: Container(
                    alignment: Alignment.centerRight,
                    child: Obx(() => Switch(
                        value: isIsNationalLegal.value,
                        onChanged: isIsNationalLegal))))
          ],
        )
      ],
    );
  }

  Widget _createSelectAdjustedDayView(
      PageController pageController, Rx<DateTime> adjustedDay) {
    return Column(
      children: [
        Row(
          children: [
            SIconButton(
                toolTip: '返回',
                onPressed: () => {
                      pageController.animateToPage(0,
                          duration: const Duration(milliseconds: 200),
                          curve: Curves.linear)
                    },
                icon: const Icon(Icons.arrow_back)),
            const SizedBox(
              width: 4,
            ),
            const Text('选择调班日期'),
          ],
        ),
        const SizedBox(height: 4),
        Expanded(
            child: CalendarDatePicker(
                initialDate: adjustedDay.value,
                firstDate: adjustedDay.value.add(const Duration(days: -365)),
                lastDate: adjustedDay.value.add(const Duration(days: 365)),
                onDateChanged: (day) {
                  adjustedDay.value = day;
                  pageController.animateToPage(0,
                      duration: const Duration(milliseconds: 200),
                      curve: Curves.linear);
                }))
      ],
    );
  }

  void _showCreateHolidyDialog(HolidayController controller, DateTime day) {
    var titleController = TextEditingController();
    var pageController = PageController();
    RxBool isAdjusted = RxBool(false);
    RxBool isIsNationalLegal = RxBool(false);
    Rx<DateTime> adjustedDay = Rx(day);
    Dialogs.showChildDialog2(
        SizedBox(
            width: 300,
            height: 320,
            child: Column(
              children: [
                const Text(
                  '创建节假日',
                  style: TextStyle(fontSize: 26),
                ),
                const Divider(),
                Expanded(
                    child: PageView(
                  controller: pageController,
                  physics: const NeverScrollableScrollPhysics(),
                  children: [
                    _createHolidayInfoView(titleController, pageController, day,
                        adjustedDay, isAdjusted, isIsNationalLegal),
                    _createSelectAdjustedDayView(pageController, adjustedDay)
                  ],
                )),
              ],
            )),
        actions: [
          SButton(
              onPressed: () => {
                    _addHoliday(
                        controller,
                        titleController.text,
                        day,
                        isAdjusted.value,
                        isIsNationalLegal.value,
                        adjustedDay.value)
                  },
              child: const Text('确定')),
          SButton(
              onPressed: () => {Dialogs.closeCurrentDialog()},
              child: const Text('取消'))
        ]);
  }

  Future _loadHolidays(HolidayController hc) async {
    EasyLoading.show(status: '正在加载节假日列表...');
    try {
      await hc.load2YearsHolidays();
      EasyLoading.dismiss();
    } catch (e) {
      EasyLoading.dismiss();
      Dialogs.showErrorDialog('加载节假日列表失败: $e');
    }
  }
}
