import 'package:dio/dio.dart';

class RpcError extends Error {
  String message;
  RpcError(this.message);

  @override
  String toString() {
    return message;
  }
}

class JsonRpc {
  final Dio _dio = Dio();

  Future<dynamic> call(
      String host, String service, String method, List args) async {
    var request = <String, dynamic>{};
    request['jsonrpc'] = '2.0';
    request['id'] = 1;
    request['method'] = method;
    if (args.isNotEmpty) {
      if (args.length == 1) {
        request['params'] = args[0];
      } else {
        request['params'] = args;
      }
    }
    var url = host + service;
    return await _postRequest(url, request);
  }

  Future<dynamic> _postRequest(String url, dynamic data) async {
    var response = await _dio.post<Map<String, dynamic>>(url, data: data);
    if (response.data != null) {
      var responseData = response.data;
      if (responseData != null) {
        var version = responseData['jsonrpc'];
        if (version != '2.0') {
          throw RpcError('通讯协议错误。');
        }
        var result = responseData['result'];
        if (result != null) {
          return result;
        } else {
          var error = responseData['error'];
          if (error != null) {
            var message = error['message'].toString();
            throw RpcError(message);
          }
        }
      } else {
        throw RpcError('通讯协议错误。');
      }
    }
  }
}
