import 'dart:io' as io;
import 'dart:html';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';

class LayoutUtils {
  static BuildContext? _context;

  static const double layoutWidth = 1920;

  static const double layoutHeight = 1080;

  static double screenWidth = 1280;

  static double screenHeight = 720;

  static RxDouble width = 1920.0.obs;

  static RxDouble height = (1080.0 - kToolbarHeight).obs;

  static double aspectRatio = width.value / height.value;

  static RxBool isDesktopMode = true.obs;

  static void setContext(BuildContext context) {
    _context = context;
    var screen = window.screen;
    if (screen != null) {
      var width = screen.width;
      var height = screen.height;
      if (width != null) {
        screenWidth = width.toDouble();
      }
      if (height != null) {
        screenHeight = height.toDouble();
      }
    }
  }

  static void updateMode() {
    if (_context == null) {
      return;
    }
    var queryData = MediaQuery.of(_context as BuildContext);
    var appSize = queryData.size;
    width.value = appSize.width;
    height.value = appSize.height - kToolbarHeight;
    aspectRatio = width.value / height.value;
    var ratio = appSize.width / appSize.height;
    if (kIsWeb) {
      isDesktopMode.value = aspectRatio > 1;
    } else {
      var isMobileDevice = io.Platform.isAndroid || io.Platform.isIOS;
      if (isMobileDevice) {
        isDesktopMode.value = ratio > 0.8;
      } else {
        isDesktopMode.value = appSize.width > 480 && appSize.height > 360;
      }
    }
  }
}
