import 'package:flutter/material.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';
import 'package:sss/layout.dart';

import 'common.dart';
import 'controllers.dart';
import 'controls.dart';
import 'dialogs.dart';

class BackgroundPainter1 extends CustomPainter {
  @override
  void paint(Canvas canvas, Size size) {
    final paint = Paint()..color = Colors.green.withAlpha(160);
    canvas.drawCircle(const Offset(0, 0), 150, paint);
  }

  @override
  bool shouldRepaint(covariant CustomPainter oldDelegate) {
    return true;
  }
}

class BackgroundPainter2 extends CustomPainter {
  @override
  void paint(Canvas canvas, Size size) {
    final paint = Paint()..color = Colors.green.withAlpha(160);
    canvas.drawCircle(Offset(size.width, size.height), 600, paint);
  }

  @override
  bool shouldRepaint(covariant CustomPainter oldDelegate) {
    return true;
  }
}

class BackgroundPainter3 extends CustomPainter {
  @override
  void paint(Canvas canvas, Size size) {
    final paint = Paint()..color = Colors.green.withAlpha(160);
    canvas.drawCircle(Offset(size.width, size.height), 300, paint);
  }

  @override
  bool shouldRepaint(covariant CustomPainter oldDelegate) {
    return true;
  }
}

class LoginView extends StatelessWidget {
  PageController pageController;
  LoginView({Key? key, required this.pageController}) : super(key: key);

  Future _login(LoginController controller) async {
    EasyLoading.show(status: '登录中...');
    try {
      await controller.login();
      pageController.animateToPage(1,
          duration: const Duration(milliseconds: 300), curve: Curves.linear);
      EasyLoading.dismiss();
    } catch (e) {
      EasyLoading.dismiss();
      Dialogs.showErrorDialog('登录失败:\n${e.toString()}');
    }
  }

  @override
  Widget build(context) {
    Common.setPageTitle(context, 'VINNO工时统计系统');
    var controller = ControllerManager.find<LoginController>();
    return Scaffold(
        resizeToAvoidBottomInset: false,
        body: Stack(alignment: Alignment.center, children: [
          CustomPaint(
              size: Size(LayoutUtils.width.value, LayoutUtils.height.value),
              foregroundPainter: BackgroundPainter1()),
          CustomPaint(
              size: Size(LayoutUtils.width.value, LayoutUtils.height.value),
              foregroundPainter: LayoutUtils.isDesktopMode.value
                  ? BackgroundPainter2()
                  : BackgroundPainter3()),
          Row(
            children: [
              LayoutUtils.isDesktopMode.value
                  ? Expanded(
                      flex: 2,
                      child: Container(
                        alignment: Alignment.center,
                        child: Column(
                            mainAxisAlignment: MainAxisAlignment.center,
                            crossAxisAlignment: CrossAxisAlignment.center,
                            children: const [
                              Text(
                                'VINNO工时统计系统',
                                style: TextStyle(
                                    fontSize: 36, fontWeight: FontWeight.bold),
                              ),
                              SizedBox(height: 30),
                              SizedBox(
                                  width: 812,
                                  child: Text(
                                    '基于Flutter/Go开发的高效便捷的工时统计系统，与钉钉完美集成，提供便捷的人员管理、假期管理、工时统计、数据审核、电子签章等功能。助力企业员工绩效管理，提高HR工作效率。',
                                    textAlign: TextAlign.center,
                                    style: TextStyle(
                                        color: Colors.grey, fontSize: 18),
                                  )),
                              SizedBox(height: 30),
                              Image(
                                  width: 640,
                                  image:
                                      AssetImage('lib/images/background.png'))
                            ]),
                      ))
                  : const SizedBox(),
              Expanded(
                  child: Container(
                      alignment: Alignment.center,
                      child: Container(
                          padding: const EdgeInsets.all(20.0),
                          decoration: BoxDecoration(
                            boxShadow: const [
                              BoxShadow(
                                  color: Colors.black12,
                                  offset: Offset(0, 15),
                                  blurRadius: 15,
                                  spreadRadius: 1)
                            ],
                            border: Border.all(
                                color: const Color.fromARGB(255, 233, 234, 239),
                                width: 2),
                            color: Colors.white,
                            borderRadius:
                                const BorderRadius.all(Radius.circular(10)),
                          ),
                          width: 360,
                          height: 360,
                          alignment: Alignment.center,
                          child: Column(
                              mainAxisAlignment: MainAxisAlignment.center,
                              children: [
                                Row(
                                  mainAxisAlignment: MainAxisAlignment.center,
                                  children: const [
                                    Text(
                                      '管理员登录',
                                      style: TextStyle(
                                          fontSize: 26,
                                          fontWeight: FontWeight.bold),
                                    )
                                  ],
                                ),
                                const SizedBox(height: 20),
                                STextField(
                                    placeHolder: '用户名',
                                    onChanged: controller.accountName,
                                    onSubmitted: (value) =>
                                        {_login(controller)}),
                                const SizedBox(height: 10),
                                STextField(
                                  placeHolder: '密码',
                                  onChanged: controller.password,
                                  obscureText: true,
                                  onSubmitted: (value) => {_login(controller)},
                                ),
                                const SizedBox(height: 10),
                                SizedBox(
                                    width: 400,
                                    height: 42,
                                    child: SButton(
                                      child: const Text('登录'),
                                      onPressed: () => {_login(controller)},
                                    ))
                              ]))))
            ],
          )
        ]));
  }
}
