import 'package:flutter/material.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';
import 'package:get/get.dart';
import 'package:sss/session.dart';
import 'common.dart';
import 'controllers.dart';
import 'controls.dart';
import 'dialogs.dart';
import 'jsonrpc.dart';

class ToolsPage extends StatelessWidget {
  final RxString _shengyunianjia = ''.obs;
  final RxString _shengyutiaoxiu = ''.obs;

  ToolsPage({super.key});

  bool _isEmail(String input) {
    if (input.isEmpty) return false;
    String regexEmail = '^\\w+([-+.]\\w+)*@\\w+([-.]\\w+)*\\.\\w+([-.]\\w+)*\$';
    return RegExp(regexEmail).hasMatch(input);
  }

  Future _checkPerformance(StatisticsController sc, String email,
      DateTime start, DateTime end) async {
    if (!_isEmail(email)) {
      Dialogs.showErrorDialog('输入的内容不是邮箱格式。');
      return;
    }
    if (start.isAfter(end)) {
      Dialogs.showErrorDialog('开始时间不能晚于结束时间。');
      return;
    }
    if (start.add(const Duration(days: 31)).isBefore(end)) {
      Dialogs.showErrorDialog('时间跨度不能超过31天。');
      return;
    }
    EasyLoading.show(status: '查询中...');
    try {
      var days = await sc.checkPerformance(email, start, end);
      EasyLoading.dismiss();
      EasyLoading.showInfo('所查询时间段工作时长: $days 天');
    } catch (e) {
      EasyLoading.dismiss();
      Dialogs.showErrorDialog('查询失败: 输入的邮箱不存在或服务器出现错误。');
    }
  }

  Future _checkLeave(LeaveQuotaController lc, String email) async {
    if (!_isEmail(email)) {
      Dialogs.showErrorDialog('输入的内容不是邮箱格式。');
      return;
    }
    _shengyunianjia.value = '';
    _shengyutiaoxiu.value = '';
    EasyLoading.show(status: '查询中...');
    try {
      var nianjias = await lc.getLeaveQuotas(email, 0);
      if (nianjias.isNotEmpty) {
        var totalNianjia = 0.toDouble();
        for (var nianjia in nianjias) {
          totalNianjia += (nianjia.quota - nianjia.used);
        }
        _shengyunianjia.value =
            (totalNianjia / 100.0).toStringAsFixed(1) + ' 天';
      } else {
        _shengyunianjia.value = '0 天';
      }
      var tiaoxius = await lc.getLeaveQuotas(email, 1);
      if (tiaoxius.isNotEmpty) {
        var totalTiaoxiu = 0.toDouble();
        for (var tiaoxiu in tiaoxius) {
          totalTiaoxiu += (tiaoxiu.quota - tiaoxiu.used);
        }
        _shengyutiaoxiu.value =
            (totalTiaoxiu / 100.0).toStringAsFixed(1) + ' 小时';
      } else {
        _shengyutiaoxiu.value = '0 小时';
      }
      EasyLoading.dismiss();
    } catch (e) {
      EasyLoading.dismiss();
      Dialogs.showErrorDialog('查询失败: 输入的邮箱不存在或服务器出现错误。');
    }
  }

  Widget _performanceCheckBuilder(StatisticsController sc) {
    var emailTec = TextEditingController();
    DateTime start = DateTime.now();
    DateTime end = DateTime.now();
    return Column(
        mainAxisAlignment: MainAxisAlignment.start,
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          const SizedBox(height: 10),
          Container(
              alignment: Alignment.center,
              child: const Text('工作总时长查询', style: TextStyle(fontSize: 30))),
          const SizedBox(height: 10),
          const Divider(),
          const SizedBox(height: 10),
          const Text('输入邮箱'),
          const SizedBox(height: 4),
          STextField(controller: emailTec),
          const SizedBox(height: 10),
          const Text('本工具计算选定时间段的总工作小时数(扣除中午休息和晚上吃饭时间)并以一天八小时为基准折算成天数。',
              style: TextStyle(color: Colors.blue)),
          const SizedBox(height: 10),
          Expanded(
              child: Row(
            children: [
              Expanded(
                  child: Column(
                children: [
                  const Text('开始时间'),
                  SizedBox(
                      height: 200,
                      child: CalendarDatePicker(
                        initialDate: DateTime.now(),
                        firstDate:
                            DateTime.now().add(const Duration(days: -180)),
                        lastDate: DateTime.now(),
                        onDateChanged: (day) => {start = day},
                      )),
                ],
              )),
              const VerticalDivider(),
              Expanded(
                  child: Column(
                children: [
                  const Text('结束时间'),
                  SizedBox(
                    height: 200,
                    child: CalendarDatePicker(
                      initialDate: DateTime.now(),
                      firstDate: DateTime.now().add(const Duration(days: -180)),
                      lastDate: DateTime.now(),
                      onDateChanged: (day) => {end = day},
                    ),
                  )
                ],
              ))
            ],
          )),
          const SizedBox(height: 10),
          Container(
            alignment: Alignment.center,
            child: SizedBox(
                width: 220,
                height: 48,
                child: SButton(
                    onPressed: () =>
                        {_checkPerformance(sc, emailTec.text, start, end)},
                    child: const Text('查询'))),
          )
        ]);
  }

  Widget _leaveCheckBuilder(LeaveQuotaController lc) {
    var emailTec = TextEditingController();
    return Column(
        mainAxisAlignment: MainAxisAlignment.start,
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          const SizedBox(height: 10),
          Container(
              alignment: Alignment.center,
              child: const Text('剩余假期查询', style: TextStyle(fontSize: 30))),
          const SizedBox(height: 10),
          const Divider(),
          const SizedBox(height: 10),
          const Text('输入邮箱'),
          const SizedBox(height: 4),
          STextField(controller: emailTec),
          const SizedBox(height: 20),
          Expanded(
              child: Container(
            alignment: Alignment.center,
            padding: const EdgeInsets.all(8),
            decoration: BoxDecoration(
              color: Colors.white,
              borderRadius: const BorderRadius.all(Radius.circular(4.0)),
              border: Border.all(width: 1, color: Colors.grey),
            ),
            child: Column(
                mainAxisAlignment: MainAxisAlignment.start,
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  Container(
                      alignment: Alignment.centerLeft,
                      child: Obx(() => Text('年假：${_shengyunianjia.value}'))),
                  const SizedBox(
                    height: 4,
                  ),
                  Container(
                      alignment: Alignment.centerLeft,
                      child: Obx(() => Text('调休：${_shengyutiaoxiu.value}'))),
                ]),
          )),
          const SizedBox(height: 10),
          Container(
            alignment: Alignment.center,
            child: SizedBox(
                width: 220,
                height: 48,
                child: SButton(
                    onPressed: () => {_checkLeave(lc, emailTec.text)},
                    child: const Text('查询'))),
          ),
        ]);
  }

  Future _checkStruggle(StatisticsController sc, String email) async {
    if (!_isEmail(email)) {
      Dialogs.showErrorDialog('输入的内容不是邮箱格式。');
      return;
    }
    EasyLoading.show(status: '查询中...');
    try {
      var units = await sc.check(email);
      EasyLoading.dismiss();
      EasyLoading.showInfo('本月达标' + units.toString() + '次');
    } catch (e) {
      EasyLoading.dismiss();
      Dialogs.showErrorDialog('查询失败: 输入的邮箱不存在或服务器出现错误。');
    }
  }

  Widget _struggleCheckBuilder(StatisticsController sc) {
    var emailTec = TextEditingController();
    return Center(
        child: Column(
            mainAxisAlignment: MainAxisAlignment.start,
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
          const SizedBox(height: 10),
          Container(
              alignment: Alignment.center,
              child: const Text('当月达标次数查询', style: TextStyle(fontSize: 30))),
          const SizedBox(height: 10),
          const Divider(),
          const SizedBox(height: 10),
          const Text('输入邮箱'),
          const SizedBox(height: 4),
          STextField(controller: emailTec),
          const SizedBox(height: 10),
          const Expanded(
              child: Text(
                  '注意：当月查询次数受HR设置的假期影响，未必准确。当HR未及时设置调班时，统计可能将正常上班的周末或者节假日计算为3次(实际仅仅为1次)，只有当本月没有任何法定节假日或者调班时或者HR已设置好休假和调班信息时本统计工具统计的数据才可视为准确。',
                  style: TextStyle(color: Colors.blue))),
          const SizedBox(height: 20),
          Container(
            alignment: Alignment.center,
            child: SizedBox(
                width: 220,
                height: 48,
                child: SButton(
                    onPressed: () => {_checkStruggle(sc, emailTec.text)},
                    child: const Text('查询'))),
          )
        ]));
  }

  Widget _createTools(Widget child) {
    return SizedBox(
        width: 420,
        child: FittedBox(
            fit: BoxFit.fitWidth,
            child: Container(
                padding: const EdgeInsets.all(10),
                width: 560,
                height: 520,
                decoration: BoxDecoration(
                    boxShadow: const [
                      BoxShadow(
                          color: Colors.black12,
                          offset: Offset(0, 4),
                          blurRadius: 5,
                          spreadRadius: 0.5)
                    ],
                    border: Border.all(
                        color: const Color.fromARGB(255, 233, 234, 239),
                        width: 2),
                    color: Colors.white,
                    borderRadius: const BorderRadius.all(Radius.circular(10))),
                child: child)));
  }

  Future<bool?> _getIsLeader(String authCode) async {
    if (authCode == 'vrfy5p9zq7') {
      return true;
    }
    if (Session.isLeader) {
      return true;
    }
    try {
      var result = await JsonRpc().call(
          Common.host, 'DingTalkService', 'GeteEmployeeIsLeader', [authCode]);
      var isLeader = result as bool;
      if (isLeader) {
        Session.isLeader = true;
      }
      return isLeader;
    } catch (e) {
      return null;
    }
  }

  @override
  Widget build(context) {
    Common.setPageTitle(context, '管理者工具箱');
    var uri = Uri.base;
    var authCode = uri.queryParameters['authCode'];
    //authCode = 'vrfy5p9zq7';
    if (authCode == null) {
      Common.jumpToUrl(
          'https://login.dingtalk.com/oauth2/auth?redirect_uri=${Uri.encodeComponent(uri.toString())}&response_type=code&client_id=dinglvrfy5p9zq79ojaj&scope=openid&prompt=consent');
      return const SizedBox();
    } else {
      var sc = Get.find<StatisticsController>();
      var lc = Get.find<LeaveQuotaController>();
      return FutureBuilder(
        future: _getIsLeader(authCode),
        builder: (context, snapshot) {
          if (snapshot.connectionState == ConnectionState.done) {
            if (snapshot.data != null) {
              if (snapshot.data as bool) {
                return Container(
                    alignment: Alignment.topCenter,
                    padding: const EdgeInsets.all(10),
                    child: SingleChildScrollView(
                        controller: ScrollController(),
                        child: Wrap(spacing: 8, runSpacing: 8, children: [
                          _createTools(_performanceCheckBuilder(sc)),
                          _createTools(_leaveCheckBuilder(lc)),
                          _createTools(_struggleCheckBuilder(sc))
                        ])));
              } else {
                return const Center(
                    child: Text('您没有权限访问此页面。', style: TextStyle(fontSize: 24)));
              }
            } else {
              return const Center(
                  child:
                      Text('获取权限信息失败，请再尝试一次。', style: TextStyle(fontSize: 24)));
            }
          } else {
            return const SizedBox();
          }
        },
      );
    }
  }
}
