import 'dart:convert';

import 'package:file_selector/file_selector.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';
import 'package:flutter_typeahead/flutter_typeahead.dart';
import 'package:get/get.dart';
import 'package:intl/intl.dart';
import 'package:qr_flutter/qr_flutter.dart';
import 'package:sss/controllers.dart';
import 'package:sss/controls.dart';
import 'package:sss/entities.dart';
import 'package:sss/layout.dart';
import 'package:sss/session.dart';
import 'package:uuid/uuid.dart';
import 'dart:math';

import 'common.dart';
import 'dialogs.dart';
import 'jsonrpc.dart';
import 'viewmodels.dart';

class ViewStatisticsPage extends StatelessWidget {
  final _isDesktopMode = true.obs;
  final _employeeFilterContrtoller = TextEditingController();
  final _departmentFilterController = TextEditingController();
  final _isShowingHelp = false.obs;

  ViewStatisticsPage({Key? key}) : super(key: key);

  @override
  Widget build(context) {
    _isDesktopMode.value = LayoutUtils.isDesktopMode.value;
    LayoutUtils.isDesktopMode.listen((value) {
      Future.delayed(const Duration(milliseconds: 10)).then((value) {
        _isDesktopMode.value = LayoutUtils.isDesktopMode.value;
      });
    });
    var fromViewCode = false;
    var uri = Uri.base;
    var viewCode = uri.queryParameters['viewCode'];
    var authCode = uri.queryParameters['authCode'];
    if (viewCode != null) {
      if (authCode == null) {
        Common.jumpToUrl(
            'https://login.dingtalk.com/oauth2/auth?redirect_uri=${Uri.encodeComponent(uri.toString())}&response_type=code&client_id=dinglvrfy5p9zq79ojaj&scope=openid&prompt=consent');
        return const SizedBox();
      } else {
        fromViewCode = true;
        Common.setPageTitle(context, '达标次数核签');
        LayoutUtils.setContext(context);
        LayoutUtils.updateMode();
        return FutureBuilder(
          future: _getEmployeeByAuthCode(authCode),
          builder: (context, snapshot) {
            if (snapshot.connectionState == ConnectionState.done) {
              if (snapshot.data != null) {
                return _createViewCodeView(context, viewCode, fromViewCode);
              } else {
                return const Center(
                    child: Text('无效的登录信息。', style: TextStyle(fontSize: 24)));
              }
            } else {
              return const SizedBox();
            }
          },
        );
      }
    }
    return _createView(context, fromViewCode);
  }

  Future<Employee?> _getEmployeeByAuthCode(String autuCode) async {
    Session.currentSessionId = "ViewCode-" + const Uuid().v1();
    var controller = Get.find<EmployeeController>();
    try {
      if (Session.loginEmployee != null) {
        return Session.loginEmployee!;
      } else {
        var result = await JsonRpc().call(Common.host, 'DingTalkService',
            'GeteEmployeeNameByAuthCode', [autuCode]);
        var employee = await controller.getEmployeeByName(result as String);
        Session.isAdmin = false;
        Session.loginEmployee = employee;
        return employee;
      }
    } catch (e) {
      Session.currentSessionId = '';
      Session.loginEmployee = null;
      Session.isAdmin = false;
      return null;
    }
  }

  void _showHelpDialog() {
    if (!_isShowingHelp.value) {
      var scrollController = ScrollController();
      _isShowingHelp.value = true;
      Dialogs.showChildDialog2(
          SizedBox(
              width: LayoutUtils.width.value / 1.5,
              height: LayoutUtils.height.value - 50,
              child: Scrollbar(
                  controller: scrollController,
                  thumbVisibility: true,
                  child: SingleChildScrollView(
                    controller: ScrollController(),
                    child: Image.network(
                        'https://static-1300984704.cos.ap-shanghai.myqcloud.com/help.jpg'),
                  ))), onDismiss: () {
        _isShowingHelp.value = false;
      });
    }
  }

  Widget _createViewCodeView(
      BuildContext context, String code, bool fromViewCode) {
    var controller = Get.find<StatisticsController>();
    return FutureBuilder(
        future: _loadViewCode(controller, code),
        builder: (context, snapshot) {
          if (snapshot.hasData) {
            if (snapshot.data as bool == true) {
              RxBool selectAll = RxBool(false);
              RxBool showQuickBar = RxBool(false);
              listenSelectedChanged(controller, showQuickBar);
              Future.delayed(const Duration(seconds: 1)).then((value) {
                _showHelpDialog();
              });
              return Scaffold(
                  resizeToAvoidBottomInset: false,
                  appBar: _createAppBar(controller),
                  body: Column(
                    children: [
                      Obx(() => _isDesktopMode.value
                          ? SizedBox(
                              height: 60,
                              child: _createOperationBar(controller))
                          : const SizedBox(height: 8)),
                      SizedBox(
                        child: Row(children: [
                          const SizedBox(
                            width: 4,
                          ),
                          Obx(
                            () => Checkbox(
                                value: selectAll.value,
                                onChanged: (value) {
                                  selectAll.value = value as bool;
                                  _selectUnselectAll(controller, value);
                                }),
                          ),
                          Expanded(
                              child: Container(
                            height: 0.5,
                            color: Colors.grey,
                          ))
                        ]),
                      ),
                      Expanded(
                          child:
                              _createWorkingDayStatisticsListView(controller)),
                      Container(
                        height: 0.5,
                        color: Colors.grey,
                      ),
                      Obx(() => showQuickBar.value
                          ? Container(
                              height: 84,
                              alignment: Alignment.center,
                              child: Row(
                                mainAxisAlignment: MainAxisAlignment.center,
                                crossAxisAlignment: CrossAxisAlignment.center,
                                children: [
                                  _createViewCodeLinkButton(controller)
                                ],
                              ),
                            )
                          : const SizedBox())
                    ],
                  ),
                  floatingActionButton: Session.loginEmployee!.isReviewer
                      ? FloatingActionButton(
                          onPressed: () => {_submitFinish(controller)},
                          child: const Icon(Icons.done),
                          tooltip: '提交审核完成通知',
                        )
                      : null);
            } else {
              return const Center(
                  child: Text('无效的访问链接, 或链接已经过期。',
                      style: TextStyle(fontSize: 24)));
            }
          } else {
            return const Center();
          }
        });
  }

  void _submitFinish(StatisticsController controller) {
    Dialogs.showConfirmDialog(
        '确定提交审核结果？点击确定会将【已完成工时统计的审核】通知到管理员。\n如你还未完成所有的审核, 请勿点击确定, 避免管理员将未审核的数据计入最终统计结果。',
        () async {
      EasyLoading.show(status: '正在提交...');
      try {
        var ac = AdminController();
        var ec = EmployeeController();
        var adminIds = await ac.getAdminIds();
        var sessionName = Session.loginEmployee != null
            ? Session.loginEmployee!.name
            : Session.currentAccountName;
        for (var adminId in adminIds) {
          await ec.sendTextMessage(adminId, sessionName + ' 已完成工时统计的审核。');
        }
        EasyLoading.dismiss();
        EasyLoading.show(status: '提交成功!');
        await Future.delayed(const Duration(seconds: 1));
        EasyLoading.dismiss();
        Dialogs.closeCurrentDialog();
      } catch (e) {
        EasyLoading.dismiss();
        Dialogs.showErrorDialog('发送【已完成工时统计的审核】通知失败！' + e.toString());
      }
    });
  }

  Future<bool> _loadViewCode(
      StatisticsController controller, String code) async {
    EasyLoading.show(status: '页面加载中...');
    try {
      await controller.selectViewCode(code);
      EasyLoading.dismiss();
      return true;
    } catch (e) {
      EasyLoading.dismiss();
      return false;
    }
  }

  AppBar _createAppBar(StatisticsController controller) {
    return AppBar(title: const Text('统计详情'), actions: [
      SIconButton(
          toolTip: '帮助',
          onPressed: () {
            _showHelpDialog();
          },
          icon: const Icon(Icons.help)),
      Obx(() => _isDesktopMode.value
          ? const SizedBox()
          : PopupMenuButton(
              tooltip: '过滤',
              itemBuilder: ((c) {
                return [
                  PopupMenuItem(
                    child: const Text('按姓名过滤员工'),
                    onTap: () =>
                        Future.delayed(const Duration(milliseconds: 10))
                            .then((value) => _showEmployeeFilter(controller)),
                  ),
                  PopupMenuItem(
                    child: const Text('按部门过滤员工'),
                    onTap: () =>
                        Future.delayed(const Duration(milliseconds: 10))
                            .then((value) => _showDepartmentFilter(controller)),
                  ),
                  PopupMenuItem(
                    child: const Text('清空过滤器'),
                    onTap: () => _clearFilter(controller),
                  )
                ];
              }),
              icon: const Icon(
                Icons.filter,
                color: Colors.white,
              ),
            ))
    ]);
  }

  void _showEmployeeFilter(StatisticsController controller) {
    Dialogs.showChildDialog2(
        SizedBox(
            width: 320,
            height: 40,
            child: STextField(
                placeHolder: '输入员工名关键字',
                controller: _employeeFilterContrtoller)),
        actions: [
          SButton(
              onPressed: () {
                controller.employeeFilter = _employeeFilterContrtoller.text;
                controller.filterStatisticsList();
                Dialogs.closeCurrentDialog();
              },
              child: const Text('过滤')),
          SButton(
              onPressed: () => {Dialogs.closeCurrentDialog()},
              child: const Text('取消')),
        ]);
  }

  void _showDepartmentFilter(StatisticsController controller) {
    Dialogs.showChildDialog2(
        SizedBox(
            width: 320,
            height: 40,
            child: STextField(
                placeHolder: '输入部门关键字',
                controller: _departmentFilterController)),
        actions: [
          SButton(
              onPressed: () {
                controller.departmentFilter = _departmentFilterController.text;
                controller.filterStatisticsList();
                Dialogs.closeCurrentDialog();
              },
              child: const Text('过滤')),
          SButton(
              onPressed: () => {Dialogs.closeCurrentDialog()},
              child: const Text('取消')),
        ]);
  }

  void _clearFilter(StatisticsController controller) {
    _employeeFilterContrtoller.clear();
    _departmentFilterController.clear();
    controller.reset();
  }

  Widget _createView(BuildContext context, bool fromViewCode) {
    if (Get.arguments == null) {
      Get.toNamed('/');
      return const SizedBox();
    }
    var arguments = Get.arguments as Map;
    if (arguments["controller"] == null) {
      Get.toNamed('/');
      return const SizedBox();
    }
    RxBool selectAll = RxBool(false);
    RxBool showQuickBar = RxBool(false);
    var controller = arguments["controller"] as StatisticsController;
    listenSelectedChanged(controller, showQuickBar);
    return Scaffold(
        appBar: _createAppBar(controller),
        body: Column(
          children: [
            Obx(() => _isDesktopMode.value
                ? SizedBox(height: 60, child: _createOperationBar(controller))
                : const SizedBox(height: 8)),
            SizedBox(
              child: Row(children: [
                const SizedBox(
                  width: 4,
                ),
                Obx(
                  () => Checkbox(
                      value: selectAll.value,
                      onChanged: (value) {
                        selectAll.value = value as bool;
                        _selectUnselectAll(controller, value);
                      }),
                ),
                Expanded(
                    child: Container(
                  height: 0.5,
                  color: Colors.grey,
                ))
              ]),
            ),
            Expanded(child: _createWorkingDayStatisticsListView(controller)),
            Container(
              height: 0.5,
              color: Colors.grey,
            ),
            Obx(() => showQuickBar.value
                ? Container(
                    height: 84,
                    alignment: Alignment.center,
                    child: Row(
                      mainAxisAlignment: MainAxisAlignment.center,
                      crossAxisAlignment: CrossAxisAlignment.center,
                      children: [
                        _createSendButton(controller),
                        const SizedBox(width: 16),
                        _createExportButton(controller),
                        const SizedBox(width: 16),
                        _createViewCodeLinkButton(controller)
                      ],
                    ),
                  )
                : const SizedBox())
          ],
        ));
  }

  Widget _createViewCodeLinkButton(StatisticsController controller) {
    return Column(
      mainAxisAlignment: MainAxisAlignment.center,
      crossAxisAlignment: CrossAxisAlignment.center,
      children: [
        SizedBox(
          width: 40,
          height: 40,
          child: FloatingActionButton(
            onPressed: () =>
                {_showShareSelectedWorkingDayStatisticsListDialog(controller)},
            child: const Icon(Icons.share),
          ),
        ),
        const SizedBox(
          height: 2,
        ),
        const Text('分享')
      ],
    );
  }

  Widget _createSendButton(StatisticsController controller) {
    return Column(
      mainAxisAlignment: MainAxisAlignment.center,
      crossAxisAlignment: CrossAxisAlignment.center,
      children: [
        SizedBox(
          width: 40,
          height: 40,
          child: FloatingActionButton(
            onPressed: () =>
                _showSendSelectedWorkingDayStatisticsListDialog(controller),
            child: const Icon(Icons.send),
          ),
        ),
        const SizedBox(
          height: 2,
        ),
        const Text('发送')
      ],
    );
  }

  Widget _createExportButton(StatisticsController controller) {
    return Column(
      mainAxisAlignment: MainAxisAlignment.center,
      crossAxisAlignment: CrossAxisAlignment.center,
      children: [
        SizedBox(
          width: 40,
          height: 40,
          child: FloatingActionButton(
            onPressed: () => _createAndDownloadReport(controller),
            child: const Icon(Icons.import_export),
          ),
        ),
        const SizedBox(
          height: 2,
        ),
        const Text('导出')
      ],
    );
  }

  Future _createAndDownloadReport(StatisticsController controller) async {
    EasyLoading.show(status: '正在生成Excel统计表...');
    try {
      var reportContent = await controller.createAndDownloadReport();
      var buffer = base64Decode(reportContent);
      var path = await getSavePath();
      if (path != null) {
        var reportName = 'Report';
        if (controller.selectedTable != null) {
          reportName = controller.selectedTable!.title;
        }
        var name = reportName + ".xlsx";
        var mimeType =
            "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet";
        final file = XFile.fromData(buffer, name: name, mimeType: mimeType);
        await file.saveTo(path);
      }
      EasyLoading.dismiss();
    } catch (e) {
      EasyLoading.dismiss();
      Dialogs.showErrorDialog('正在生成Excel统计表失败: ${e.toString()}');
    }
  }

  void listenSelectedChanged(
      StatisticsController controller, RxBool showQuickBar) {
    for (var workingDayStatistics
        in controller.workingDayStatisticsList.value) {
      workingDayStatistics.isSelected.listen((selected) {
        if (selected) {
          showQuickBar.value = true;
        } else {
          if (!controller.workingDayStatisticsList.value
              .any((element) => element.isSelected.value)) {
            showQuickBar.value = false;
          }
        }
      });
    }
  }

  Future _createAndSendViewCode(
      StatisticsController sc,
      List<Employee>? reviewers,
      String receivers,
      String ccs,
      String contentTemplate,
      DateTime expiredTime) async {
    if (reviewers == null) {
      Dialogs.showErrorDialog('不存在有效的审核人员邮箱。');
      return;
    }
    var emails = receivers.split(';');
    if (emails.isEmpty) {
      Dialogs.showErrorDialog('请选择有效的审核人员邮箱。');
      return;
    }
    for (var email in emails) {
      if (email != '' && !email.isEmail) {
        Dialogs.showErrorDialog('错误的审核人员邮箱格式。');
        return;
      }
    }
    if (ccs != "") {
      var ccList = ccs.split(';');
      if (ccList.isEmpty) {
        Dialogs.showErrorDialog('请选择有效的抄送人员邮箱。');
        return;
      }
      for (var cc in ccList) {
        if (cc != '' && !cc.isEmail) {
          Dialogs.showErrorDialog('错误的抄送人员邮箱格式。');
          return;
        }
      }
    }
    var selected = sc.workingDayStatisticsList.value
        .where((element) => element.isSelected.value)
        .toList()
        .map((e) => e.data.id)
        .toList();
    if (selected.isEmpty) {
      Dialogs.showErrorDialog('请选择要发送的统计表内容。');
      return;
    }
    EasyLoading.show(status: '正在创建和发送统计列表共享码');
    try {
      for (var email in emails) {
        if (email != '') {
          var reviewer =
              reviewers.firstWhereOrNull((element) => element.email == email);
          if (reviewer != null) {
            var uuid = const Uuid();
            var id = uuid.v1();
            var code = DateFormat('yyMMdd').format(DateTime.now()) +
                uuid.v1().split('-')[0];
            var viewCode =
                ViewCode(id, code, reviewer.id, selected, expiredTime);
            await sc.createAndSendViewCode(
                email, ccs, contentTemplate, viewCode);
            await Future.delayed(const Duration(seconds: 1));
          }
        }
      }
      EasyLoading.dismiss();
      EasyLoading.showInfo('邮件发送成功!');
      EasyLoading.dismiss();
      Dialogs.closeCurrentDialog();
    } catch (e) {
      EasyLoading.dismiss();
      Dialogs.showErrorDialog('创建和发送统计列表共享码失败: $e');
    }
  }

  Future _getMessageTemplate(MessageTemplateController mc) async {
    EasyLoading.show(status: '正在加载消息模板...');
    try {
      await mc.getMessageTemplate();
      EasyLoading.dismiss();
    } catch (e) {
      EasyLoading.dismiss();
      Dialogs.showErrorDialog('读取消息模板失败: $e');
    }
  }

  Future<String> _createViewCode(StatisticsController controller) async {
    var selected = controller.workingDayStatisticsList.value
        .where((element) => element.isSelected.value)
        .toList()
        .map((e) => e.data.id)
        .toList();
    if (selected.isEmpty) {
      Dialogs.showErrorDialog('请选择要分享的统计记录。');
      return '';
    }
    EasyLoading.show(status: '正在创建共享码...');
    try {
      var now = DateTime.now();
      var uuid = const Uuid();
      var code = DateFormat('yyMMdd').format(now) + uuid.v1().split('-')[0];
      var id = uuid.v1();
      var viewCode =
          ViewCode(id, code, '', selected, now.add(const Duration(days: 7)));
      await controller.createViewCode(viewCode);
      var domainController = Get.find<DomainController>();
      if (domainController.setting == null) {
        await domainController.getDomainSetting();
      }
      if (domainController.setting == null) {
        throw Exception('系统域名未设置!');
      }
      await EasyLoading.dismiss();
      return '${domainController.setting!.domain}/viewstatistcs?viewCode=$code';
    } catch (e) {
      EasyLoading.dismiss();
      Dialogs.showErrorDialog('创建共享码失败: $e');
      return '';
    }
  }

  Future _showShareSelectedWorkingDayStatisticsListDialog(
      StatisticsController controller) async {
    var code = await _createViewCode(controller);
    if (code != '') {
      Dialogs.showChildDialog2(SizedBox(
        width: 240,
        height: 240,
        child: Column(children: [
          const SizedBox(
            height: 10,
          ),
          QrImage(
            data: code,
            version: QrVersions.auto,
            size: 200.0,
          ),
          const SizedBox(
            width: 10,
          ),
          SButton(
              onPressed: () => {Clipboard.setData(ClipboardData(text: code))},
              child: const Text('复制链接'))
        ]),
      ));
    }
  }

  Future _showSendSelectedWorkingDayStatisticsListDialog(
      StatisticsController controller) async {
    var mc = MessageTemplateController();
    await _getMessageTemplate(mc);
    EmployeeController ec = EmployeeController();
    var reviewerController = TextEditingController();
    var ccController = TextEditingController();
    var content = 'Dear, 请打开链接审核加班统计结果： {URL} 并于 {ExpiredTime} 前完成审核和提交。';
    var template = mc.templates
        .firstWhereOrNull((element) => element.name == 'ReviewMessageTemplate');
    if (template != null) {
      content = template.content;
    }
    var contentTemplateController = TextEditingController(text: content);
    var pageController = PageController();
    var expiredTime = Rx(DateTime.now().add(const Duration(days: 7)));
    List<Employee>? reviewers;
    Dialogs.showChildDialog2(
        SizedBox(
            width: 480,
            height: 520,
            child: Column(
                mainAxisAlignment: MainAxisAlignment.start,
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  Container(
                      width: 512,
                      alignment: Alignment.center,
                      child: const Text(
                        '发送邮件',
                        style: TextStyle(fontSize: 26),
                      )),
                  const Divider(),
                  Expanded(
                      child: PageView(
                    physics: const NeverScrollableScrollPhysics(),
                    controller: pageController,
                    children: [
                      Column(
                        children: [
                          TypeAheadField(
                              textFieldConfiguration: TextFieldConfiguration(
                                  controller: reviewerController,
                                  decoration: DefaultInputDecoration(
                                      placeHolder: '输入审核人员邮箱')),
                              suggestionsCallback: (pattern) async {
                                reviewers ??= await ec.loadReviewers();
                                var lastPattern = pattern.split(';').last;
                                return reviewers!.where((element) =>
                                    element.email.contains(lastPattern));
                              },
                              itemBuilder: (context, suggestion) {
                                var reviewer = suggestion as Employee;
                                return ListTile(
                                    leading: const Icon(Icons.person),
                                    title: Text(
                                        '${reviewer.name} (${reviewer.email})'));
                              },
                              onSuggestionSelected: (suggestion) {
                                var reviewer = suggestion as Employee;
                                if (reviewerController.text.isNotEmpty) {
                                  var lastChar = reviewerController
                                      .text[reviewerController.text.length - 1];
                                  if (lastChar == ';') {
                                    reviewerController.text +=
                                        reviewer.email + ';';
                                  } else {
                                    var lastIndex = reviewerController.text
                                        .lastIndexOf(';');
                                    if (lastIndex == -1) {
                                      reviewerController.text =
                                          reviewer.email + ';';
                                    } else {
                                      reviewerController.text =
                                          reviewerController.text
                                                  .substring(0, lastIndex) +
                                              ';' +
                                              reviewer.email +
                                              ';';
                                    }
                                  }
                                } else {
                                  reviewerController.text =
                                      reviewer.email + ';';
                                }
                              }),
                          const SizedBox(height: 10),
                          TypeAheadField(
                              textFieldConfiguration: TextFieldConfiguration(
                                  controller: ccController,
                                  decoration: DefaultInputDecoration(
                                      placeHolder: '输入抄送人员邮箱')),
                              suggestionsCallback: (pattern) async {
                                reviewers ??= await ec.loadReviewers();
                                var lastPattern = pattern.split(';').last;
                                return reviewers!.where((element) =>
                                    element.email.contains(lastPattern));
                              },
                              itemBuilder: (context, suggestion) {
                                var reviewer = suggestion as Employee;
                                return ListTile(
                                    leading: const Icon(Icons.person),
                                    title: Text(
                                        '${reviewer.name} (${reviewer.email})'));
                              },
                              onSuggestionSelected: (suggestion) {
                                var reviewer = suggestion as Employee;
                                if (ccController.text.isNotEmpty) {
                                  var lastChar = ccController
                                      .text[ccController.text.length - 1];
                                  if (lastChar == ';') {
                                    ccController.text += reviewer.email + ';';
                                  } else {
                                    var lastIndex =
                                        ccController.text.lastIndexOf(';');
                                    if (lastIndex == -1) {
                                      ccController.text = reviewer.email + ';';
                                    } else {
                                      ccController.text = ccController.text
                                              .substring(0, lastIndex) +
                                          ';' +
                                          reviewer.email +
                                          ';';
                                    }
                                  }
                                } else {
                                  ccController.text = reviewer.email + ';';
                                }
                              }),
                          const SizedBox(height: 10),
                          Container(
                              alignment: Alignment.centerLeft,
                              child: const Text('邮件正文')),
                          const SizedBox(height: 2),
                          STextField(
                              controller: contentTemplateController,
                              maxLines: 12),
                          const SizedBox(height: 10),
                          Row(
                            children: [
                              const Text('过期时间'),
                              Expanded(
                                  child: Container(
                                alignment: Alignment.centerRight,
                                child: Row(
                                  mainAxisAlignment: MainAxisAlignment.end,
                                  children: [
                                    Obx(() => Text(DateFormat('yyyy-MM-dd')
                                        .format(expiredTime.value))),
                                    const SizedBox(
                                      width: 4,
                                    ),
                                    SIconButton(
                                      toolTip: '选择过期时间',
                                      onPressed: () =>
                                          {pageController.jumpToPage(1)},
                                      icon: const Icon(Icons.arrow_forward_ios),
                                    )
                                  ],
                                ),
                              ))
                            ],
                          )
                        ],
                      ),
                      Column(
                        children: [
                          Container(
                            alignment: Alignment.centerLeft,
                            child: SIconButton(
                              toolTip: '返回',
                              onPressed: () => {pageController.jumpToPage(0)},
                              icon: const Icon(Icons.arrow_back),
                            ),
                          ),
                          const SizedBox(height: 10),
                          Container(
                              alignment: Alignment.centerLeft,
                              child: const Text('过期时间')),
                          SizedBox(
                              height: 240,
                              child: CalendarDatePicker(
                                initialDate: expiredTime.value,
                                firstDate:
                                    DateTime.now().add(const Duration(days: 1)),
                                lastDate:
                                    DateTime.now().add(const Duration(days: 7)),
                                onDateChanged: (day) {
                                  expiredTime.value = day;
                                  pageController.jumpToPage(0);
                                },
                              )),
                        ],
                      )
                    ],
                  ))
                ])),
        actions: [
          SButton(
              onPressed: () => {
                    _createAndSendViewCode(
                        controller,
                        reviewers,
                        reviewerController.text,
                        ccController.text,
                        contentTemplateController.text,
                        expiredTime.value)
                  },
              child: const Text('发送')),
          SButton(
              onPressed: () => {Dialogs.closeCurrentDialog()},
              child: const Text('取消'))
        ]);
  }

  void _selectUnselectAll(StatisticsController controller, bool? selectAll) {
    var value = selectAll ?? false;
    if (controller.filterMode.value) {
      for (var workingDayStatistics
          in controller.filteredWorkingDayStatisticsList.value) {
        workingDayStatistics.isSelected.value = value;
      }
    } else {
      for (var workingDayStatistics
          in controller.workingDayStatisticsList.value) {
        workingDayStatistics.isSelected.value = value;
      }
    }
  }

  Widget _createDepartmentNameFilter(StatisticsController controller) {
    return TypeAheadField(
        textFieldConfiguration: TextFieldConfiguration(
            controller: _departmentFilterController,
            decoration: DefaultInputDecoration(placeHolder: '输入部门名过滤')),
        suggestionsCallback: (pattern) async {
          return controller.currentDepartmentNameList.where((element) =>
              element.toLowerCase().contains(pattern.toLowerCase()));
        },
        itemBuilder: (context, suggestion) {
          return ListTile(
              leading: const Icon(Icons.group),
              title: Text(suggestion as String));
        },
        onSuggestionSelected: (suggestion) {
          _departmentFilterController.text = suggestion as String;
        });
  }

  Widget _createOperationBar(StatisticsController controller) {
    return Row(
      children: [
        const SizedBox(
          width: 10,
        ),
        SizedBox(
            width: 300,
            height: 48,
            child: STextField(
              controller: _employeeFilterContrtoller,
              placeHolder: '输入员工名过滤',
              onSubmitted: (key) {
                controller.employeeFilter = _employeeFilterContrtoller.text;
                controller.filterStatisticsList();
              },
            )),
        const SizedBox(
          width: 10,
        ),
        SizedBox(
          width: 300,
          height: 48,
          child: _createDepartmentNameFilter(controller),
        ),
        const SizedBox(
          width: 10,
        ),
        SComButton(
            onPressed: () {
              controller.employeeFilter = _employeeFilterContrtoller.text;
              controller.departmentFilter = _departmentFilterController.text;
              controller.filterStatisticsList();
            },
            icon: const Icon(Icons.search),
            label: const Text('过滤')),
        const SizedBox(
          width: 10,
        ),
        SComButton(
            onPressed: () {
              _employeeFilterContrtoller.clear();
              _departmentFilterController.clear();
              controller.reset();
            },
            icon: const Icon(Icons.refresh),
            label: const Text('重置')),
      ],
    );
  }

  Widget _createEmployeeNameCell(
      WorkingDayStatisticsViewModel workingDayStatistics) {
    return Row(
        mainAxisAlignment: MainAxisAlignment.start,
        crossAxisAlignment: CrossAxisAlignment.center,
        children: [
          Checkbox(
              value: workingDayStatistics.isSelected.value,
              onChanged: (value) =>
                  {workingDayStatistics.isSelected.value = value as bool}),
          const SizedBox(
            width: 4,
          ),
          _createEmployeeReachedFlag(workingDayStatistics),
          const SizedBox(
            width: 8,
          ),
          Column(
            mainAxisAlignment: MainAxisAlignment.center,
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              Row(
                children: [
                  Text(workingDayStatistics.data.employee.name),
                  const SizedBox(
                    width: 4,
                  ),
                  _createCommentFlag(workingDayStatistics)
                ],
              ),
              Text(
                workingDayStatistics.data.departmentName,
                style: const TextStyle(fontSize: 10, color: Colors.grey),
              )
            ],
          ),
        ]);
  }

  Widget _createCommentFlag(
      WorkingDayStatisticsViewModel workingDayStatistics) {
    if ((workingDayStatistics.adjusted.value &&
            workingDayStatistics.adjustDescription.value != '' &&
            workingDayStatistics.adjustDescription.value != 'null') ||
        (workingDayStatistics.comment.value != '' &&
            workingDayStatistics.data.comment != 'null')) {
      var comment = '';
      if (workingDayStatistics.adjusted.value &&
          workingDayStatistics.adjustDescription.value != '' &&
          workingDayStatistics.adjustDescription.value != 'null') {
        comment += '审核修改说明:\r\n' + workingDayStatistics.adjustDescription.value;
      }
      if (workingDayStatistics.comment.value != '' &&
          workingDayStatistics.data.comment != 'null') {
        if (comment != '') {
          comment += '\r\n\r\n';
        }
        comment += '备注:\r\n' + workingDayStatistics.comment.value;
      }

      return Container(
          alignment: Alignment.center,
          child: Tooltip(
            child: const Icon(Icons.info),
            padding: const EdgeInsets.all(6),
            textStyle: const TextStyle(fontSize: 14, color: Colors.white),
            message: comment,
          ));
    } else {
      return const SizedBox();
    }
  }

  Widget _createEmployeeReachedFlag(
      WorkingDayStatisticsViewModel workingDayStatistics) {
    return Row(children: [
      workingDayStatistics.adjustedReachedCount.value <
              workingDayStatistics.data.requiredReachedCount
          ? SizedBox(
              width: 48,
              height: 48,
              child: Stack(
                alignment: Alignment.center,
                children: [
                  const Icon(
                    Icons.data_usage,
                    color: Colors.red,
                    size: 48,
                  ),
                  Text(
                    '${workingDayStatistics.adjustedReachedCount.value}',
                    textAlign: TextAlign.center,
                    style: const TextStyle(fontSize: 12, color: Colors.red),
                  )
                ],
              ),
            )
          : SizedBox(
              width: 48,
              height: 48,
              child: Stack(
                alignment: Alignment.center,
                children: [
                  const Icon(
                    Icons.brightness_low,
                    color: Colors.green,
                    size: 48,
                  ),
                  Text(
                    '${workingDayStatistics.adjustedReachedCount.value}',
                    textAlign: TextAlign.center,
                    style: const TextStyle(fontSize: 12, color: Colors.green),
                  )
                ],
              ),
            ),
    ]);
  }

  Future _viewAndEditWorkingDayStatistics(StatisticsController sc,
      WorkingDayStatisticsViewModel workingDayStatistics) async {
    EasyLoading.show(status: '正在加载工作日信息...');
    try {
      await sc.selectWorkingDayStatistics(workingDayStatistics);
      EasyLoading.dismiss();
      await Get.toNamed('/viewworkingdays', arguments: {'sc': sc});
    } catch (e) {
      EasyLoading.dismiss();
      Dialogs.showErrorDialog('加载工作日信息失败: ${e.toString()}');
    }
  }

  Future _finishWorkingDayStatistics(StatisticsController controller,
      WorkingDayStatisticsViewModel workingDayStatistics) async {
    EasyLoading.show(status: '正在完成审核...');
    try {
      if (Session.loginEmployee != null) {
        workingDayStatistics.data.reviewer = Session.loginEmployee!.name;
      } else {
        workingDayStatistics.data.reviewer = Session.currentAccountName;
      }

      await controller.updateWorkingDayStatistics(workingDayStatistics.data);

      if (Session.loginEmployee != null) {
        workingDayStatistics.reviewer.value = Session.loginEmployee!.name;
      } else {
        workingDayStatistics.reviewer.value = Session.currentAccountName;
      }
      EasyLoading.dismiss();
    } catch (e) {
      EasyLoading.dismiss();
      Dialogs.showErrorDialog('完成审核失败: ${e.toString()}');
    }
  }

  Widget _createEnployeeStatisticsView(StatisticsController controller,
      WorkingDayStatisticsViewModel workingDayStatistics) {
    return Obx(() => Row(children: [
          _createEmployeeNameCell(workingDayStatistics),
          Expanded(
              child: Container(
                  padding: EdgeInsets.zero,
                  alignment: Alignment.centerRight,
                  child: workingDayStatistics.reviewer.value == ''
                      ? SComButton(
                          onPressed: () {
                            _finishWorkingDayStatistics(
                                controller, workingDayStatistics);
                          },
                          icon: const Icon(Icons.approval),
                          label: const Text('核签'))
                      : Transform.rotate(
                          angle: -pi / 6,
                          child: Container(
                            alignment: Alignment.center,
                            width: 96,
                            height: 36,
                            decoration: BoxDecoration(
                                border: Border.all(color: Colors.red, width: 1),
                                borderRadius: BorderRadius.circular(8)),
                            padding: const EdgeInsets.all(2),
                            child: Container(
                              alignment: Alignment.center,
                              decoration: BoxDecoration(
                                  border:
                                      Border.all(color: Colors.red, width: 1),
                                  borderRadius: BorderRadius.circular(6)),
                              child: Text(
                                workingDayStatistics.reviewer.value,
                                textAlign: TextAlign.center,
                                style: const TextStyle(
                                    color: Colors.red, fontSize: 10),
                              ),
                            ),
                          )))),
          const SizedBox(
            width: 4,
          )
        ]));
  }

  Widget _createWorkingDayStatisticsListView(StatisticsController controller) {
    return Obx(() => ListView.builder(
        controller: ScrollController(),
        padding: EdgeInsets.zero,
        itemCount: controller.filterMode.value
            ? controller.filteredWorkingDayStatisticsList.value.length
            : controller.workingDayStatisticsList.value.length,
        itemExtent: 64,
        itemBuilder: (BuildContext context, int index) {
          var workingDayStatistics = controller.filterMode.value
              ? controller.filteredWorkingDayStatisticsList.value[index]
              : controller.workingDayStatisticsList.value[index];
          return GestureDetector(
              onTap: () {
                _viewAndEditWorkingDayStatistics(
                    controller, workingDayStatistics);
              },
              child: !_isDesktopMode.value
                  ? Container(
                      color: Colors.white10,
                      margin: const EdgeInsets.all(4),
                      child: _createEnployeeStatisticsView(
                          controller, workingDayStatistics))
                  : MouseRegion(
                      cursor: SystemMouseCursors.click,
                      onHover: (event) =>
                          workingDayStatistics.isHovering.value = true,
                      onExit: (event) =>
                          workingDayStatistics.isHovering.value = false,
                      child: Obx(
                        () => workingDayStatistics.isHovering.value
                            ? Container(
                                margin: const EdgeInsets.all(4),
                                decoration: BoxDecoration(
                                  border: Border.all(
                                      color: const Color.fromARGB(
                                          255, 233, 234, 239),
                                      width: 2),
                                  color:
                                      const Color.fromARGB(255, 242, 242, 242),
                                  borderRadius: const BorderRadius.all(
                                      Radius.circular(2)),
                                ),
                                child: _createEnployeeStatisticsView(
                                    controller, workingDayStatistics))
                            : Container(
                                margin: const EdgeInsets.all(4),
                                child: _createEnployeeStatisticsView(
                                    controller, workingDayStatistics)),
                      ),
                    ));
        }));
  }
}
